//
// XML Set functions
//

#include "CvGameCoreDLL.h"
#include "CvDLLXMLIFaceBase.h"
#include "CvXMLLoadUtility.h"
#include "CvGlobals.h"
#include "CvArtFileMgr.h"
#include "CvGameTextMgr.h"
#include <algorithm>
#include "CvInfoWater.h"
#include "FProfiler.h"
#include "FVariableSystem.h"
#include "CvGameCoreUtils.h"

// Macro for Setting Global Art Defines
#define INIT_ART_DEFINE_XML_LOAD(xmlFile, xmlInfoPath, infoArray, numInfos) SetGlobalArtDefineInfo(xmlFile, xmlInfoPath, infoArray, numInfos);
#define INIT_XML_GLOBAL_LOAD(xmlInfoPath, infoArray, numInfos)  SetGlobalClassInfo(infoArray, xmlInfoPath, numInfos);

bool CvXMLLoadUtility::ReadGlobalDefines(const TCHAR* szXMLFileName, CvCacheObject* cache)
{
	bool bLoaded = false;	// used to make sure that the xml file was loaded correctly

	if (!gDLL->cacheRead(cache, szXMLFileName))			// src data file name
	{
		// load normally
		if (!CreateFXml())
		{
			return false;
		}

		// load the new FXml variable with the szXMLFileName file
		bLoaded = LoadCivXml(m_pFXml, szXMLFileName);
		if (!bLoaded)
		{
			char	szMessage[1024];
			sprintf( szMessage, "LoadXML call failed for %s \n Current XML file is: %s", szXMLFileName, GC.getCurrentXMLFile().GetCString());
			gDLL->MessageBox(szMessage, "XML Load Error");
		}

		// if the load succeeded we will continue
		if (bLoaded)
		{
			// if the xml is successfully validated
			if (Validate())
			{
				// locate the first define tag in the xml
				if (gDLL->getXMLIFace()->LocateNode(m_pFXml,"Civ4Defines/Define"))
				{
					int i;	// loop counter
					// get the number of other Define tags in the xml file
					int iNumDefines = gDLL->getXMLIFace()->GetNumSiblings(m_pFXml);
					// add one to the total in order to include the current Define tag
					iNumDefines++;

					// loop through all the Define tags
					for (i=0;i<iNumDefines;i++)
					{
						char szNodeType[256];	// holds the type of the current node
						char szName[256];

						// Skip any comments and stop at the next value we might want
						if (SkipToNextVal())
						{
							// call the function that sets the FXml pointer to the first non-comment child of
							// the current tag and gets the value of that new node
							if (GetChildXmlVal(szName))
							{
								// set the FXml pointer to the next sibling of the current tag``
								if (gDLL->getXMLIFace()->NextSibling(GetXML()))
								{
									// Skip any comments and stop at the next value we might want
									if (SkipToNextVal())
									{
										// if we successfuly get the node type for the current tag
										if (gDLL->getXMLIFace()->GetLastLocatedNodeType(GetXML(),szNodeType))
										{
											// if the node type of the current tag isn't null
											if (strcmp(szNodeType,"")!=0)
											{
												// if the node type of the current tag is a float then
												if (strcmp(szNodeType,"float")==0)
												{
													// get the float value for the define
													float fVal;
													GetXmlVal(&fVal);
													GC.getDefinesVarSystem()->SetValue(szName, fVal);
												}
												// else if the node type of the current tag is an int then
												else if (strcmp(szNodeType,"int")==0)
												{
													// get the int value for the define
													int iVal;
													GetXmlVal(&iVal);
													GC.getDefinesVarSystem()->SetValue(szName, iVal);
												}
												// else if the node type of the current tag is a boolean then
												else if (strcmp(szNodeType,"boolean")==0)   // FIX for boolean Global Defines by Impaler[WrG]
												{
													// get the boolean value for the define
													bool bVal;
													GetXmlVal(&bVal);
													GC.getDefinesVarSystem()->SetValue(szName, bVal);
												}
												// otherwise we will assume it is a string/text value
												else
												{
													char szVal[256];
													// get the string/text value for the define
													GetXmlVal(szVal);
													GC.getDefinesVarSystem()->SetValue(szName, szVal);
												}
											}
											// otherwise we will default to getting the string/text value for the define
											else
											{
												char szVal[256];
												// get the string/text value for the define
												GetXmlVal(szVal);
												GC.getDefinesVarSystem()->SetValue(szName, szVal);
											}
										}
									}
								}

								// since we are looking at the children of a Define tag we will need to go up
								// one level so that we can go to the next Define tag.
								// Set the FXml pointer to the parent of the current tag
								gDLL->getXMLIFace()->SetToParent(GetXML());
							}
						}

						// now we set the FXml pointer to the sibling of the current tag, which should be the next
						// Define tag
						if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
						{
							break;
						}
					}

					// write global defines info to cache
					bool bOk = gDLL->cacheWrite(cache);
					if (!bOk)
					{
						char	szMessage[1024];
						sprintf( szMessage, "Failed writing to global defines cache. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
						gDLL->MessageBox(szMessage, "XML Caching Error");
					}
					else
					{
						logMsg("Wrote GlobalDefines to cache");
					}
				}
			}
		}

		// delete the pointer to the FXml variable
		gDLL->getXMLIFace()->DestroyFXml(m_pFXml);
	}
	else
	{
		logMsg("Read GobalDefines from cache");
	}

	return true;
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetGlobalDefines()
//
//  PURPOSE :   Initialize the variables located in globaldefines.cpp/h with the values in
//				GlobalDefines.xml
//
//------------------------------------------------------------------------------------------------------
bool CvXMLLoadUtility::SetGlobalDefines()
{
	UpdateProgressCB("GlobalDefines");

	/////////////////////////////////
	//
	// use disk cache if possible.
	// if no cache or cache is older than xml file, use xml file like normal, else read from cache
	//

	CvCacheObject* cache = gDLL->createGlobalDefinesCacheObject("GlobalDefines.dat");	// cache file name

	if (!ReadGlobalDefines("xml\\GlobalDefines.xml", cache))
	{
		return false;
	}

	if (!ReadGlobalDefines("xml\\GlobalDefinesAlt.xml", cache))
	{
		return false;
	}

    // Start Modular GlobalDefines by Impaler[WrG]
    std::vector<CvString> files;
    gDLL->enumerateFiles(files, "xml\\*_GlobalDefines.xml");
    int i;
    bool failure = false;
    for(i=0;i<(int)files.size();i++)
    {
        if (!ReadGlobalDefines(files[i], cache))
        {
            failure = true;
        }
    }
    if (failure)
    {
        return false;
    }
    // End Modular GlobalDefines by Impaler[WrG]

	gDLL->destroyCache(cache);
	////////////////////////////////////////////////////////////////////////

	GC.cacheGlobals();

	return true;
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetPostGlobalsGlobalDefines()
//
//  PURPOSE :   This function assumes that the SetGlobalDefines function has already been called
//							it then loads the few global defines that needed to reference a global variable that
//							hadn't been loaded in prior to the SetGlobalDefines call
//
//------------------------------------------------------------------------------------------------------
bool CvXMLLoadUtility::SetPostGlobalsGlobalDefines()
{
	const char* szVal=NULL;		// holds the string value from the define queue
	int idx;

	if (GC.getDefinesVarSystem()->GetSize() > 0)
	{
		SetGlobalDefine("LAND_TERRAIN", szVal);
		idx = FindInInfoClass(szVal, GC.getTerrainInfo(), sizeof(GC.getTerrainInfo((TerrainTypes) 0)), GC.getNumTerrainInfos());
		GC.getDefinesVarSystem()->SetValue("LAND_TERRAIN", idx);

		SetGlobalDefine("DEEP_WATER_TERRAIN", szVal);
		idx = FindInInfoClass(szVal, GC.getTerrainInfo(), sizeof(GC.getTerrainInfo((TerrainTypes) 0)), GC.getNumTerrainInfos());
		GC.getDefinesVarSystem()->SetValue("DEEP_WATER_TERRAIN", idx);

		SetGlobalDefine("SHALLOW_WATER_TERRAIN", szVal);
		idx = FindInInfoClass(szVal, GC.getTerrainInfo(), sizeof(GC.getTerrainInfo((TerrainTypes) 0)), GC.getNumTerrainInfos());
		GC.getDefinesVarSystem()->SetValue("SHALLOW_WATER_TERRAIN", idx);

		SetGlobalDefine("LAND_IMPROVEMENT", szVal);
		idx = FindInInfoClass(szVal, GC.getImprovementInfo(), sizeof(GC.getImprovementInfo((ImprovementTypes) 0)), GC.getNumImprovementInfos());
		GC.getDefinesVarSystem()->SetValue("LAND_IMPROVEMENT", idx);

		SetGlobalDefine("WATER_IMPROVEMENT", szVal);
		idx = FindInInfoClass(szVal, GC.getImprovementInfo(), sizeof(GC.getImprovementInfo((ImprovementTypes) 0)), GC.getNumImprovementInfos());
		GC.getDefinesVarSystem()->SetValue("WATER_IMPROVEMENT", idx);

		SetGlobalDefine("RUINS_IMPROVEMENT", szVal);
		idx = FindInInfoClass(szVal, GC.getImprovementInfo(), sizeof(GC.getImprovementInfo((ImprovementTypes) 0)), GC.getNumImprovementInfos());
		GC.getDefinesVarSystem()->SetValue("RUINS_IMPROVEMENT", idx);

		SetGlobalDefine("NUKE_FEATURE", szVal);
		idx = FindInInfoClass(szVal, GC.getFeatureInfo(), sizeof(GC.getFeatureInfo((FeatureTypes) 0)), GC.getNumFeatureInfos());
		GC.getDefinesVarSystem()->SetValue("NUKE_FEATURE", idx);

		SetGlobalDefine("GLOBAL_WARMING_TERRAIN", szVal);
		idx = FindInInfoClass(szVal, GC.getTerrainInfo(), sizeof(GC.getTerrainInfo((TerrainTypes) 0)), GC.getNumTerrainInfos());
		GC.getDefinesVarSystem()->SetValue("GLOBAL_WARMING_TERRAIN", idx);

		SetGlobalDefine("CAPITAL_BUILDINGCLASS", szVal);
		idx = FindInInfoClass(szVal, GC.getBuildingClassInfo(), sizeof(GC.getBuildingClassInfo((BuildingClassTypes) 0)), GC.getNumBuildingClassInfos());
		GC.getDefinesVarSystem()->SetValue("CAPITAL_BUILDINGCLASS", idx);

		SetGlobalDefine("DEFAULT_SPECIALIST", szVal);
		idx = FindInInfoClass(szVal, GC.getSpecialistInfo(), sizeof(GC.getSpecialistInfo((SpecialistTypes) 0)), GC.getNumSpecialistInfos());
		GC.getDefinesVarSystem()->SetValue("DEFAULT_SPECIALIST", idx);

		SetGlobalDefine("INITIAL_CITY_ROUTE_TYPE", szVal);
		idx = FindInInfoClass(szVal, GC.getRouteInfo(), sizeof(GC.getRouteInfo((RouteTypes) 0)), GC.getNumRouteInfos());
		GC.getDefinesVarSystem()->SetValue("INITIAL_CITY_ROUTE_TYPE", idx);

		SetGlobalDefine("STANDARD_HANDICAP", szVal);
		idx = FindInInfoClass(szVal, GC.getHandicapInfo(), sizeof(GC.getHandicapInfo((HandicapTypes) 0)), GC.getNumHandicapInfos());
		GC.getDefinesVarSystem()->SetValue("STANDARD_HANDICAP", idx);

		SetGlobalDefine("STANDARD_HANDICAP_QUICK", szVal);
		idx = FindInInfoClass(szVal, GC.getHandicapInfo(), sizeof(GC.getHandicapInfo((HandicapTypes) 0)), GC.getNumHandicapInfos());
		GC.getDefinesVarSystem()->SetValue("STANDARD_HANDICAP_QUICK", idx);

		SetGlobalDefine("STANDARD_GAMESPEED", szVal);
		idx = FindInInfoClass(szVal, GC.getGameSpeedInfo(), sizeof(GC.getGameSpeedInfo((GameSpeedTypes) 0)), GC.getNumGameSpeedInfos());
		GC.getDefinesVarSystem()->SetValue("STANDARD_GAMESPEED", idx);

		SetGlobalDefine("STANDARD_TURNTIMER", szVal);
		idx = FindInInfoClass(szVal, GC.getTurnTimerInfo(), sizeof(GC.getTurnTimerInfo((TurnTimerTypes) 0)), GC.getNumTurnTimerInfos());
		GC.getDefinesVarSystem()->SetValue("STANDARD_TURNTIMER", idx);

		SetGlobalDefine("STANDARD_CLIMATE", szVal);
		idx = FindInInfoClass(szVal, GC.getClimateInfo(), sizeof(GC.getClimateInfo((ClimateTypes) 0)), GC.getNumClimateInfos());
		GC.getDefinesVarSystem()->SetValue("STANDARD_CLIMATE", idx);

		SetGlobalDefine("STANDARD_SEALEVEL", szVal);
		idx = FindInInfoClass(szVal, GC.getSeaLevelInfo(), sizeof(GC.getSeaLevelInfo((SeaLevelTypes) 0)), GC.getNumSeaLevelInfos());
		GC.getDefinesVarSystem()->SetValue("STANDARD_SEALEVEL", idx);

		SetGlobalDefine("STANDARD_ERA", szVal);
		idx = FindInInfoClass(szVal, GC.getEraInfo(), sizeof(GC.getEraInfo((EraTypes) 0)), GC.getNumEraInfos());
		GC.getDefinesVarSystem()->SetValue("STANDARD_ERA", idx);

		SetGlobalDefine("STANDARD_CALENDAR", szVal);
		idx = FindInInfoClass(szVal, GC.getCalendarInfo(), sizeof(GC.getCalendarInfo((CalendarTypes) 0)), GC.getNumCalendarInfos());
		GC.getDefinesVarSystem()->SetValue("STANDARD_CALENDAR", idx);

		SetGlobalDefine("AI_HANDICAP", szVal);
		idx = FindInInfoClass(szVal, GC.getHandicapInfo(), sizeof(GC.getHandicapInfo((HandicapTypes) 0)), GC.getNumHandicapInfos());
		GC.getDefinesVarSystem()->SetValue("AI_HANDICAP", idx);

		SetGlobalDefine("BARBARIAN_HANDICAP", szVal);
		idx = FindInInfoClass(szVal, GC.getHandicapInfo(), sizeof(GC.getHandicapInfo((HandicapTypes) 0)), GC.getNumHandicapInfos());
		GC.getDefinesVarSystem()->SetValue("BARBARIAN_HANDICAP", idx);

		SetGlobalDefine("BARBARIAN_CIVILIZATION", szVal);
		idx = FindInInfoClass(szVal, GC.getCivilizationInfo(), sizeof(GC.getCivilizationInfo((CivilizationTypes) 0)), GC.getNumCivilizationInfos());
		GC.getDefinesVarSystem()->SetValue("BARBARIAN_CIVILIZATION", idx);

		SetGlobalDefine("BARBARIAN_LEADER", szVal);
		idx = FindInInfoClass(szVal, GC.getLeaderHeadInfo(), sizeof(GC.getLeaderHeadInfo((LeaderHeadTypes) 0)), GC.getNumLeaderHeadInfos());
		GC.getDefinesVarSystem()->SetValue("BARBARIAN_LEADER", idx);

		return true;
	}

	char	szMessage[1024];
	sprintf( szMessage, "Size of Global Defines is not greater than 0. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
	gDLL->MessageBox(szMessage, "XML Load Error");

	return false;
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetGlobalTypes()
//
//  PURPOSE :   Initialize the variables located in globaltypes.cpp/h with the values in
//				GlobalTypes.xml
//
//------------------------------------------------------------------------------------------------------
bool CvXMLLoadUtility::SetGlobalTypes()
{
	UpdateProgressCB("GlobalTypes");

	bool bLoaded = false;	// used to make sure that the xml file was loaded correctly
	if (!CreateFXml())
	{
		return false;
	}

	// load the new FXml variable with the GlobalTypes.xml file
	bLoaded = LoadCivXml(m_pFXml, "GlobalTypes.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for GlobalTypes.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}

	// if the load succeeded we will continue
	if (bLoaded)
	{
		// if the xml is successfully validated
		if (Validate())
		{
			SetGlobalStringArray(&GC.getAnimationOperatorTypes(), "Civ4Types/AnimationOperatorTypes/AnimationOperatorType", &GC.getNumAnimationOperatorTypes());
			int iEnumVal = NUM_FUNC_TYPES;
			SetGlobalStringArray(&GC.getFunctionTypes(), "Civ4Types/FunctionTypes/FunctionType", &iEnumVal, true);
			SetGlobalStringArray(&GC.getFlavorTypes(), "Civ4Types/FlavorTypes/FlavorType", &GC.getNumFlavorTypes());
			SetGlobalStringArray(&GC.getArtStyleTypes(), "Civ4Types/ArtStyleTypes/ArtStyleType", &GC.getNumArtStyleTypes());
			SetGlobalStringArray(&GC.getCitySizeTypes(), "Civ4Types/CitySizeTypes/CitySizeType", &GC.getNumCitySizeTypes());
			iEnumVal = NUM_CONTACT_TYPES;
			SetGlobalStringArray(&GC.getContactTypes(), "Civ4Types/ContactTypes/ContactType", &iEnumVal, true);
			iEnumVal = NUM_DIPLOMACYPOWER_TYPES;
			SetGlobalStringArray(&GC.getDiplomacyPowerTypes(), "Civ4Types/DiplomacyPowerTypes/DiplomacyPowerType", &iEnumVal, true);
			iEnumVal = NUM_AUTOMATE_TYPES;
			SetGlobalStringArray(&GC.getAutomateTypes(), "Civ4Types/AutomateTypes/AutomateType", &iEnumVal, true);
			iEnumVal = NUM_DIRECTION_TYPES;
			SetGlobalStringArray(&GC.getDirectionTypes(), "Civ4Types/DirectionTypes/DirectionType", &iEnumVal, true);
			SetGlobalStringArray(&GC.getFootstepAudioTypes(), "Civ4Types/FootstepAudioTypes/FootstepAudioType", &GC.getNumFootstepAudioTypes());

			gDLL->getXMLIFace()->SetToParent(m_pFXml);
			gDLL->getXMLIFace()->SetToParent(m_pFXml);
			SetVariableListTagPair(&GC.getFootstepAudioTags(), "FootstepAudioTags", GC.getFootstepAudioTypes(), GC.getNumFootstepAudioTypes(), "");
		}
	}

	// delete the pointer to the FXml variable
	DestroyFXml();

	return true;
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetDiplomacyCommentTypes()
//
//  PURPOSE :   Creates a full list of Diplomacy Comments
//
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetDiplomacyCommentTypes(CvString** ppszString, int* iNumVals)
{
	FAssertMsg(false, "should never get here");
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetupGlobalLandscapeInfos()
//
//  PURPOSE :   Initialize the appropriate variables located in globals.cpp/h with the values in
//				Terrain\Civ4TerrainSettings.xml
//
//------------------------------------------------------------------------------------------------------
bool CvXMLLoadUtility::SetupGlobalLandscapeInfo()
{
	bool bLoaded = false;	// used to make sure that the xml file was loaded correctly
	if (!CreateFXml())
	{
		return false;
	}

	// load the new FXml variable with the CIV4TerrainSettings.xml file
	bLoaded = LoadCivXml(m_pFXml, "Terrain/CIV4TerrainSettings.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for GameInfo/CIV4TerrainSettings.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}

	if (bLoaded)
	{
		// if the xml is successfully validated
		if (Validate())
		{
			SetGlobalClassInfo(&GC.getLandscapeInfo(), "Civ4TerrainSettings/LandscapeInfos/LandscapeInfo", &GC.getNumLandscapeInfos());
		}
	}
	// delete the pointer to the FXml variable
	DestroyFXml();
	return true;
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetGlobalArtDefines()
//
//  PURPOSE :   Initialize the appropriate variables located in globals.cpp/h with the values in
//				Civ4ArtDefines.xml
//
//------------------------------------------------------------------------------------------------------
bool CvXMLLoadUtility::SetGlobalArtDefines()
{
		INIT_ART_DEFINE_XML_LOAD(&ARTFILEMGR.getInterfaceArtInfo(), "Art/CIV4ArtDefines_Interface.xml", "Civ4ArtDefines/InterfaceArtInfos/InterfaceArtInfo", ARTFILEMGR.getNumInterfaceArtInfos())
		INIT_ART_DEFINE_XML_LOAD(&ARTFILEMGR.getMovieArtInfo(), "Art/CIV4ArtDefines_Movie.xml", "Civ4ArtDefines/MovieArtInfos/MovieArtInfo", ARTFILEMGR.getNumMovieArtInfos())
		INIT_ART_DEFINE_XML_LOAD(&ARTFILEMGR.getMiscArtInfo(), "Art/CIV4ArtDefines_Misc.xml", "Civ4ArtDefines/MiscArtInfos/MiscArtInfo",  ARTFILEMGR.getNumMiscArtInfos())
		INIT_ART_DEFINE_XML_LOAD(&ARTFILEMGR.getUnitArtInfo(), "Art/CIV4ArtDefines_Unit.xml", "Civ4ArtDefines/UnitArtInfos/UnitArtInfo",  ARTFILEMGR.getNumUnitArtInfos())
		INIT_ART_DEFINE_XML_LOAD(&ARTFILEMGR.getBuildingArtInfo(), "Art/CIV4ArtDefines_Building.xml", "Civ4ArtDefines/BuildingArtInfos/BuildingArtInfo",  ARTFILEMGR.getNumBuildingArtInfos())
		INIT_ART_DEFINE_XML_LOAD(&ARTFILEMGR.getCivilizationArtInfo(), "Art/CIV4ArtDefines_Civilization.xml", "Civ4ArtDefines/CivilizationArtInfos/CivilizationArtInfo",  ARTFILEMGR.getNumCivilizationArtInfos())
		INIT_ART_DEFINE_XML_LOAD(&ARTFILEMGR.getLeaderheadArtInfo(), "Art/CIV4ArtDefines_Leaderhead.xml", "Civ4ArtDefines/LeaderheadArtInfos/LeaderheadArtInfo",  ARTFILEMGR.getNumLeaderheadArtInfos())
		INIT_ART_DEFINE_XML_LOAD(&ARTFILEMGR.getBonusArtInfo(), "Art/CIV4ArtDefines_Bonus.xml", "Civ4ArtDefines/BonusArtInfos/BonusArtInfo", ARTFILEMGR.getNumBonusArtInfos())
		INIT_ART_DEFINE_XML_LOAD(&ARTFILEMGR.getImprovementArtInfo(), "Art/CIV4ArtDefines_Improvement.xml", "Civ4ArtDefines/ImprovementArtInfos/ImprovementArtInfo", ARTFILEMGR.getNumImprovementArtInfos())
		INIT_ART_DEFINE_XML_LOAD(&ARTFILEMGR.getTerrainArtInfo(), "Art/CIV4ArtDefines_Terrain.xml", "Civ4ArtDefines/TerrainArtInfos/TerrainArtInfo", ARTFILEMGR.getNumTerrainArtInfos())
		INIT_ART_DEFINE_XML_LOAD(&ARTFILEMGR.getFeatureArtInfo(), "Art/CIV4ArtDefines_Feature.xml", "Civ4ArtDefines/FeatureArtInfos/FeatureArtInfo", ARTFILEMGR.getNumFeatureArtInfos())

    // Start Modular Loading by Chinese American -- added section to read extra art defines
    AddGlobalArtDefineInfo(&ARTFILEMGR.getUnitArtInfo(), "xml\\*_CIV4ArtDefines_Unit.xml", "Civ4ArtDefines/UnitArtInfos/UnitArtInfo", ARTFILEMGR.getNumUnitArtInfos());
    AddGlobalArtDefineInfo(&ARTFILEMGR.getBuildingArtInfo(), "xml\\*_CIV4ArtDefines_Building.xml", "Civ4ArtDefines/BuildingArtInfos/BuildingArtInfo", ARTFILEMGR.getNumBuildingArtInfos());
    AddGlobalArtDefineInfo(&ARTFILEMGR.getImprovementArtInfo(), "xml\\*_CIV4ArtDefines_Improvement.xml", "Civ4ArtDefines/ImprovementArtInfos/ImprovementArtInfo", ARTFILEMGR.getNumImprovementArtInfos());
    AddGlobalArtDefineInfo(&ARTFILEMGR.getCivilizationArtInfo(), "xml\\*_CIV4ArtDefines_Civilization.xml", "Civ4ArtDefines/CivilizationArtInfos/CivilizationArtInfo",  ARTFILEMGR.getNumCivilizationArtInfos());
    AddGlobalArtDefineInfo(&ARTFILEMGR.getLeaderheadArtInfo(), "xml\\*_CIV4ArtDefines_Leaderhead.xml", "Civ4ArtDefines/LeaderheadArtInfos/LeaderheadArtInfo",  ARTFILEMGR.getNumLeaderheadArtInfos());
	AddGlobalArtDefineInfo(&ARTFILEMGR.getBonusArtInfo(), "xml\\*_CIV4ArtDefines_Bonus.xml", "Civ4ArtDefines/BonusArtInfos/BonusArtInfo", ARTFILEMGR.getNumBonusArtInfos());
    // End Modular Loading by Chinese American -- added section to read extra art defines

	return true;
}


template <class T>
bool CvXMLLoadUtility::SetGlobalArtDefineInfo(T **ppArtDefneInfos, char* szXMLFileName, char* szTagName, int& iNumVals)
{
	if (!CreateFXml())
		return false;

	bool bLoaded = LoadCivXml(m_pFXml, szXMLFileName);
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Art/CIV4ArtDefines.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}

	if (bLoaded && Validate())
	{
		INIT_XML_GLOBAL_LOAD(szTagName, &*ppArtDefneInfos, &iNumVals)
	}
	DestroyFXml();
	return bLoaded;
}


//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetGlobalText()
//
//  PURPOSE :   Handles all Global Text Infos
//
//------------------------------------------------------------------------------------------------------
bool CvXMLLoadUtility::LoadGlobalText()
{
	CvCacheObject* cache = gDLL->createGlobalTextCacheObject("GlobalText.dat");	// cache file name
	if (!gDLL->cacheRead(cache))
	{
		bool bLoaded = false;

		if (!CreateFXml())
		{
			return false;
		}

		//
		// load all files in the xml text directory
		//
		std::vector<CvString> files;
		gDLL->enumerateFiles(files, "xml\\text\\*.xml");
		int i;
		for(i=0;i<(int)files.size();i++)
		{
			bLoaded = LoadCivXml(m_pFXml, files[i]);	// Load the XML
			if (!bLoaded)
			{
				char	szMessage[1024];
				sprintf( szMessage, "LoadXML call failed for %s. \n Current XML file is: %s", files[i].c_str(), GC.getCurrentXMLFile().GetCString());
				gDLL->MessageBox(szMessage, "XML Load Error");
			}
			if (bLoaded)
			{
				// if the xml is successfully validated
				if (Validate())
				{
					SetGameText("Civ4GameText", "Civ4GameText/TEXT");
				}
			}
		}

		gDLL->enumerateFiles(files, "xml\\*_CIV4GameText.xml");
        for(i=0;i<(int)files.size();i++)
        {
            bLoaded = LoadCivXml(m_pFXml, files[i]);	// Load each XML
            if (!bLoaded)
            {
                char	szMessage[1024];
                sprintf( szMessage, "Modular loading failed for GameText.xml \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
                gDLL->MessageBox(szMessage, "XML Load Error");
            }

            if (bLoaded)
			{
				// if the xml is successfully validated
				if (Validate())
				{
					SetGameText("Civ4GameText", "Civ4GameText/TEXT");
				}
			}

        }

		DestroyFXml();

		// write global text info to cache
		bool bOk = gDLL->cacheWrite(cache);
		if (!bLoaded)
		{
			char	szMessage[1024];
			sprintf( szMessage, "Failed writing to Global Text cache. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
			gDLL->MessageBox(szMessage, "XML Caching Error");
		}
		if (bOk)
		{
			logMsg("Wrote GlobalText to cache");
		}
	}	// didn't read from cache
	else
	{
		logMsg("Read GlobalText from cache");
	}

	gDLL->destroyCache(cache);

	return true;
}

bool CvXMLLoadUtility::LoadBasicInfos()
{
	if (!CreateFXml())
	{
		return false;
	}

	bool bLoaded = LoadCivXml(m_pFXml, "BasicInfos/CIV4BasicInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for BasicInfos/CIV4BasicInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}

	if (bLoaded && Validate())
	{
		SetGlobalClassInfo(&GC.getConceptInfo(), "Civ4BasicInfos/ConceptInfos/ConceptInfo", &GC.getNumConceptInfos());
		SetGlobalClassInfo(&GC.getCityTabInfo(), "Civ4BasicInfos/CityTabInfos/CityTabInfo", &GC.getNumCityTabInfos());
		SetGlobalClassInfo(&GC.getCalendarInfo(), "Civ4BasicInfos/CalendarInfos/CalendarInfo", &GC.getNumCalendarInfos());
		SetGlobalClassInfo(&GC.getSeasonInfo(), "Civ4BasicInfos/SeasonInfos/SeasonInfo", &GC.getNumSeasonInfos());
		SetGlobalClassInfo(&GC.getMonthInfo(), "Civ4BasicInfos/MonthInfos/MonthInfo", &GC.getNumMonthInfos());
		SetGlobalClassInfo(&GC.getDenialInfo(), "Civ4BasicInfos/DenialInfos/DenialInfo", &GC.getNumDenialInfos());
		SetGlobalClassInfo(&GC.getInvisibleInfo(), "Civ4BasicInfos/InvisibleInfos/InvisibleInfo", &GC.getNumInvisibleInfos());
		SetGlobalClassInfo(&GC.getUnitCombatInfo(), "Civ4BasicInfos/UnitCombatInfos/UnitCombatInfo", &GC.getNumUnitCombatInfos());

		int iEnumVal = NUM_DOMAIN_TYPES;
		SetGlobalClassInfo(&GC.getDomainInfo(), "Civ4BasicInfos/DomainInfos/DomainInfo", &iEnumVal, true);

		iEnumVal = NUM_UNITAI_TYPES;
		SetGlobalClassInfo(&GC.getUnitAIInfo(), "Civ4BasicInfos/UnitAIInfos/UnitAIInfo", &iEnumVal, true);

		iEnumVal = NUM_ATTITUDE_TYPES;
		SetGlobalClassInfo(&GC.getAttitudeInfo(), "Civ4BasicInfos/AttitudeInfos/AttitudeInfo", &iEnumVal, true);

		iEnumVal = NUM_MEMORY_TYPES;
		SetGlobalClassInfo(&GC.getMemoryInfo(), "Civ4BasicInfos/MemoryInfos/MemoryInfo", &iEnumVal, true);
	}

	DestroyFXml();
	return bLoaded;
}

//
// Globals which must be loaded before the main menus.
// Don't put anything in here unless it has to be loaded before the main menus,
// instead try to load things in LoadPostMenuGlobals()
//

bool CvXMLLoadUtility::LoadPreMenuGlobals()
{
	if (!CreateFXml())
	{
		return false;
	}

	int i;
	bool bLoaded;

    // Initialize All class types first to search for modular files and Map all the Type data

    std::vector<CvString> GameSpeed;
    InitializeClassLoading(GameSpeed, "CIV4GameSpeedInfo", "GameInfo", "Civ4GameSpeedInfo/GameSpeedInfos/GameSpeedInfo", &GC.getNumGameSpeedInfos());
    std::vector<CvString> TurnTimer;
    InitializeClassLoading(TurnTimer, "CIV4TurnTimerInfo", "GameInfo", "Civ4TurnTimerInfo/TurnTimerInfos/TurnTimerInfo", &GC.getNumTurnTimerInfos());
    std::vector<CvString> World;
    InitializeClassLoading(World, "CIV4WorldInfo", "GameInfo", "Civ4WorldInfo/WorldInfos/WorldInfo", &GC.getNumWorldInfos());
    std::vector<CvString> Climate;
    InitializeClassLoading(Climate, "CIV4ClimateInfo", "GameInfo", "Civ4ClimateInfo/ClimateInfos/ClimateInfo", &GC.getNumClimateInfos());
    std::vector<CvString> SeaLevel;
    InitializeClassLoading(SeaLevel, "CIV4SeaLevelInfo", "GameInfo", "Civ4SeaLevelInfo/SeaLevelInfos/SeaLevelInfo", &GC.getNumSeaLevelInfos());
    std::vector<CvString> Advisor;
    InitializeClassLoading(Advisor, "CIV4AdvisorInfos", "Interface", "Civ4AdvisorInfos/AdvisorInfos/AdvisorInfo", &GC.getNumAdvisorInfos());
    std::vector<CvString> Terrain;
    InitializeClassLoading(Terrain, "CIV4TerrainInfos", "Terrain", "Civ4TerrainInfos/TerrainInfos/TerrainInfo", &GC.getNumTerrainInfos());
    std::vector<CvString> Era;
    InitializeClassLoading(Era, "CIV4EraInfos", "GameInfo", "Civ4EraInfos/EraInfos/EraInfo", &GC.getNumEraInfos());
    std::vector<CvString> UnitClass;
    InitializeClassLoading(UnitClass, "CIV4UnitClassInfos", "Units", "Civ4UnitClassInfos/UnitClassInfos/UnitClassInfo", &GC.getNumUnitClassInfos());
    std::vector<CvString> Unit;
    InitializeClassLoading(Unit, "CIV4UnitInfos", "Units", "Civ4UnitInfos/UnitInfos/UnitInfo", &GC.getNumUnitInfos());
    std::vector<CvString> Specialist;
    InitializeClassLoading(Specialist, "CIV4SpecialistInfos", "GameInfo", "Civ4SpecialistInfos/SpecialistInfos/SpecialistInfo", &GC.getNumSpecialistInfos());
    std::vector<CvString> Tech;
    InitializeClassLoading(Tech, "CIV4TechInfos", "Technologies", "Civ4TechInfos/TechInfos/TechInfo", &GC.getNumTechInfos());
    std::vector<CvString> Feature;
    InitializeClassLoading(Feature, "CIV4FeatureInfos", "Terrain", "Civ4FeatureInfos/FeatureInfos/FeatureInfo", &GC.getNumFeatureInfos());
    std::vector<CvString> Promotion;
    InitializeClassLoading(Promotion, "CIV4PromotionInfos", "Units", "Civ4PromotionInfos/PromotionInfos/PromotionInfo", &GC.getNumPromotionInfos());
    std::vector<CvString> Goody;
    InitializeClassLoading(Goody, "CIV4GoodyInfo", "GameInfo", "Civ4GoodyInfo/GoodyInfos/GoodyInfo", &GC.getNumGoodyInfos());
    std::vector<CvString> Handicap;
    InitializeClassLoading(Handicap, "CIV4HandicapInfo", "GameInfo", "Civ4HandicapInfo/HandicapInfos/HandicapInfo", &GC.getNumHandicapInfos());
    std::vector<CvString> Upkeep;
    InitializeClassLoading(Upkeep, "CIV4UpkeepInfo", "GameInfo", "Civ4UpkeepInfo/UpkeepInfos/UpkeepInfo", &GC.getNumUpkeepInfos());
    std::vector<CvString> Hurry;
    InitializeClassLoading(Hurry, "CIV4HurryInfo", "GameInfo", "Civ4HurryInfo/HurryInfos/HurryInfo", &GC.getNumHurryInfos());
    std::vector<CvString> Religion;
    InitializeClassLoading(Religion, "CIV4ReligionInfo", "GameInfo", "Civ4ReligionInfo/ReligionInfos/ReligionInfo", &GC.getNumReligionInfos());
    std::vector<CvString> CultureLevel;
    InitializeClassLoading(CultureLevel, "CIV4CultureLevelInfo", "GameInfo", "Civ4CultureLevelInfo/CultureLevelInfos/CultureLevelInfo", &GC.getNumCultureLevelInfos());
    std::vector<CvString> BonusClass;
    InitializeClassLoading(BonusClass, "CIV4BonusClassInfos", "Terrain", "Civ4BonusClassInfos/BonusClassInfos/BonusClassInfo", &GC.getNumBonusClassInfos());
    std::vector<CvString> Bonus;
    InitializeClassLoading(Bonus, "CIV4BonusInfos", "Terrain", "Civ4BonusInfos/BonusInfos/BonusInfo", &GC.getNumBonusInfos());
    std::vector<CvString> BuildingClass;
    InitializeClassLoading(BuildingClass, "CIV4BuildingClassInfos", "Buildings", "Civ4BuildingClassInfos/BuildingClassInfos/BuildingClassInfo", &GC.getNumBuildingClassInfos());
    std::vector<CvString> Building;
    InitializeClassLoading(Building, "CIV4BuildingInfos", "Buildings", "Civ4BuildingInfos/BuildingInfos/BuildingInfo", &GC.getNumBuildingInfos());
    std::vector<CvString> Victory;
    InitializeClassLoading(Victory, "CIV4VictoryInfo", "GameInfo", "Civ4VictoryInfo/VictoryInfos/VictoryInfo", &GC.getNumVictoryInfos());
    std::vector<CvString> Trait;
    InitializeClassLoading(Trait, "CIV4TraitInfos", "Civilizations",  "Civ4TraitInfos/TraitInfos/TraitInfo", &GC.getNumTraitInfos());
    std::vector<CvString> CivicOption;
    InitializeClassLoading(CivicOption, "CIV4CivicOptionInfos", "GameInfo", "Civ4CivicOptionInfos/CivicOptionInfos/CivicOptionInfo", &GC.getNumCivicOptionInfos());
    std::vector<CvString> Civic;
    InitializeClassLoading(Civic, "CIV4CivicInfos", "GameInfo", "Civ4CivicInfos/CivicInfos/CivicInfo", &GC.getNumCivicInfos());
    std::vector<CvString> SpecialBuilding;
    InitializeClassLoading(SpecialBuilding, "CIV4SpecialBuildingInfos", "Buildings", "Civ4SpecialBuildingInfos/SpecialBuildingInfos/SpecialBuildingInfo", &GC.getNumSpecialBuildingInfos());
    std::vector<CvString> SpecialUnit;
    InitializeClassLoading(SpecialUnit, "CIV4SpecialUnitInfos", "Units", "Civ4SpecialUnitInfos/SpecialUnitInfos/SpecialUnitInfo", &GC.getNumSpecialUnitInfos());
    std::vector<CvString> Project;
    InitializeClassLoading(Project, "CIV4ProjectInfo", "GameInfo", "Civ4ProjectInfo/ProjectInfos/ProjectInfo", &GC.getNumProjectInfos());
    std::vector<CvString> Route;
    InitializeClassLoading(Route, "CIV4RouteInfos", "Misc", "Civ4RouteInfos/RouteInfos/RouteInfo", &GC.getNumRouteInfos());
    std::vector<CvString> Improvement;
    InitializeClassLoading(Improvement, "CIV4ImprovementInfos", "Terrain", "Civ4ImprovementInfos/ImprovementInfos/ImprovementInfo", &GC.getNumImprovementInfos());
    std::vector<CvString> Color;
    InitializeClassLoading(Color, "CIV4ColorVals", "Interface", "Civ4ColorVals/ColorVals/ColorVal", &GC.getNumColorInfos());
    std::vector<CvString> PlayerColor;
    InitializeClassLoading(PlayerColor, "CIV4PlayerColorInfos", "Interface", "Civ4PlayerColorInfos/PlayerColorInfos/PlayerColorInfo", &GC.getNumPlayerColorInfos());
    std::vector<CvString> Effect;
    InitializeClassLoading(Effect, "CIV4EffectInfos", "Misc", "Civ4EffectInfos/EffectInfos/EffectInfo", &GC.getNumEffectInfos());
    std::vector<CvString> Build;
    InitializeClassLoading(Build, "CIV4BuildInfos", "Units", "Civ4BuildInfos/BuildInfos/BuildInfo", &GC.getNumBuildInfos());
    std::vector<CvString> LeaderHead;
    InitializeClassLoading(LeaderHead, "CIV4LeaderHeadInfos", "Civilizations", "Civ4LeaderHeadInfos/LeaderHeadInfos/LeaderHeadInfo", &GC.getNumLeaderHeadInfos());
    std::vector<CvString> Civilization;
    InitializeClassLoading(Civilization, "CIV4CivilizationInfos", "Civilizations", "Civ4CivilizationInfos/CivilizationInfos/CivilizationInfo", &GC.getNumCivilizationInfos());

    // Load some weird files, hopefully these can be modularized in the future


	bLoaded = LoadCivXml( m_pFXml, "Units/CIV4AnimationInfos.xml" );
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Units/CIV4AnimationInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if ( bLoaded )
	{
		if (Validate())
		{
			SetGlobalClassInfo( &GC.getAnimationCategoryInfo(), "Civ4AnimationInfos/AnimationCategories/AnimationCategory", &GC.getNumAnimationCategoryInfos() );
			SetGlobalClassInfo( &GC.getAnimationPathInfo(), "Civ4AnimationInfos/AnimationPaths/AnimationPath", &GC.getNumAnimationPathInfos() );
			SetGlobalClassInfo( &GC.getEntityEventInfo(), "Civ4AnimationInfos/EntityEventInfos/EntityEventInfo", &GC.getNumEntityEventInfos() );
		}
	}

	// load the new FXml variable with the CIV4CursorInfo.xml file
	bLoaded = LoadCivXml(m_pFXml, "GameInfo/CIV4CursorInfo.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for GameInfo/CIV4CursorInfo.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded)
	{
		if (Validate())
		{
			SetGlobalClassInfo(&GC.getCursorInfo(), "Civ4CursorInfo/CursorInfos/CursorInfo", &GC.getNumCursorInfos());

			// add types to global var system
			int i, iVal;
			for(i=0;i<GC.getNumCursorInfos();i++)
			{
				CvString szType = GC.getCursorInfo((CursorTypes)i).getType();
				if (GC.getDefinesVarSystem()->GetValue(szType, iVal)==true)
				{
					char	szMessage[1024];
					sprintf( szMessage, "cursor type already set? \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
					gDLL->MessageBox(szMessage, "XML Error");
				}
				GC.getDefinesVarSystem()->SetValue(szType, i);
			}
		}
	}

    bLoaded = LoadCivXml(m_pFXml, "Misc/CIV4RouteInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Misc/CIV4RouteInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalRouteInfo(&GC.getRouteInfo(), "Civ4RouteInfos/RouteInfos/RouteInfo", &GC.getNumRouteInfos(), &GC.getRouteZBias());
	}

    // Complete Class loading after all classes are Initialized to get the remained fo the data loaded

    CompleteClassLoading(GameSpeed, &GC.getGameSpeedInfo(), "Civ4GameSpeedInfo/GameSpeedInfos/GameSpeedInfo", &GC.getNumGameSpeedInfos());
    CompleteClassLoading(TurnTimer, &GC.getTurnTimerInfo(), "Civ4TurnTimerInfo/TurnTimerInfos/TurnTimerInfo", &GC.getNumTurnTimerInfos());
    CompleteClassLoading(World, &GC.getWorldInfo(), "Civ4WorldInfo/WorldInfos/WorldInfo", &GC.getNumWorldInfos());
    CompleteClassLoading(Climate, &GC.getClimateInfo(), "Civ4ClimateInfo/ClimateInfos/ClimateInfo", &GC.getNumClimateInfos());
    CompleteClassLoading(SeaLevel, &GC.getSeaLevelInfo(), "Civ4SeaLevelInfo/SeaLevelInfos/SeaLevelInfo", &GC.getNumSeaLevelInfos());
    CompleteClassLoading(Advisor, &GC.getAdvisorInfo(), "Civ4AdvisorInfos/AdvisorInfos/AdvisorInfo", &GC.getNumAdvisorInfos());
    CompleteClassLoading(Terrain, &GC.getTerrainInfo(), "Civ4TerrainInfos/TerrainInfos/TerrainInfo", &GC.getNumTerrainInfos());
    CompleteClassLoading(Era, &GC.getEraInfo(), "Civ4EraInfos/EraInfos/EraInfo", &GC.getNumEraInfos());
    CompleteClassLoading(UnitClass, &GC.getUnitClassInfo(), "Civ4UnitClassInfos/UnitClassInfos/UnitClassInfo", &GC.getNumUnitClassInfos());
    CompleteClassLoading(Specialist, &GC.getSpecialistInfo(), "Civ4SpecialistInfos/SpecialistInfos/SpecialistInfo", &GC.getNumSpecialistInfos());
    CompleteClassLoading(Feature, &GC.getFeatureInfo(), "Civ4FeatureInfos/FeatureInfos/FeatureInfo", &GC.getNumFeatureInfos());
    CompleteClassLoading(Promotion, &GC.getPromotionInfo(), "Civ4PromotionInfos/PromotionInfos/PromotionInfo", &GC.getNumPromotionInfos());
    CompleteClassLoading(Goody, &GC.getGoodyInfo(), "Civ4GoodyInfo/GoodyInfos/GoodyInfo", &GC.getNumGoodyInfos());
    CompleteClassLoading(Upkeep, &GC.getUpkeepInfo(), "Civ4UpkeepInfo/UpkeepInfos/UpkeepInfo", &GC.getNumUpkeepInfos());
    CompleteClassLoading(Hurry, &GC.getHurryInfo(), "Civ4HurryInfo/HurryInfos/HurryInfo", &GC.getNumHurryInfos());
    CompleteClassLoading(Religion, &GC.getReligionInfo(), "Civ4ReligionInfo/ReligionInfos/ReligionInfo", &GC.getNumReligionInfos());
    CompleteClassLoading(CultureLevel, &GC.getCultureLevelInfo(), "Civ4CultureLevelInfo/CultureLevelInfos/CultureLevelInfo", &GC.getNumCultureLevelInfos());
    CompleteClassLoading(BonusClass, &GC.getBonusClassInfo(), "Civ4BonusClassInfos/BonusClassInfos/BonusClassInfo", &GC.getNumBonusClassInfos());
    CompleteClassLoading(BuildingClass, &GC.getBuildingClassInfo(), "Civ4BuildingClassInfos/BuildingClassInfos/BuildingClassInfo", &GC.getNumBuildingClassInfos());
    CompleteClassLoading(Victory, &GC.getVictoryInfo(), "Civ4VictoryInfo/VictoryInfos/VictoryInfo", &GC.getNumVictoryInfos());
    CompleteClassLoading(Trait, &GC.getTraitInfo(), "Civ4TraitInfos/TraitInfos/TraitInfo", &GC.getNumTraitInfos());
    CompleteClassLoading(CivicOption, &GC.getCivicOptionInfo(), "Civ4CivicOptionInfos/CivicOptionInfos/CivicOptionInfo", &GC.getNumCivicOptionInfos());
    CompleteClassLoading(SpecialBuilding, &GC.getSpecialBuildingInfo(), "Civ4SpecialBuildingInfos/SpecialBuildingInfos/SpecialBuildingInfo", &GC.getNumSpecialBuildingInfos());
    CompleteClassLoading(SpecialUnit, &GC.getSpecialUnitInfo(), "Civ4SpecialUnitInfos/SpecialUnitInfos/SpecialUnitInfo", &GC.getNumSpecialUnitInfos());
    CompleteClassLoading(Project, &GC.getProjectInfo(), "Civ4ProjectInfo/ProjectInfos/ProjectInfo", &GC.getNumProjectInfos());
    CompleteClassLoading(Color, &GC.getColorInfo(), "Civ4ColorVals/ColorVals/ColorVal", &GC.getNumColorInfos());
    CompleteClassLoading(PlayerColor,  &GC.getPlayerColorInfo(), "Civ4PlayerColorInfos/PlayerColorInfos/PlayerColorInfo", &GC.getNumPlayerColorInfos());
    CompleteClassLoading(Effect, &GC.getEffectInfo(), "Civ4EffectInfos/EffectInfos/EffectInfo", &GC.getNumEffectInfos());
    CompleteClassLoading(Build, &GC.getBuildInfo(), "Civ4BuildInfos/BuildInfos/BuildInfo", &GC.getNumBuildInfos());

    // Complete Class loading for file types that can be cached

    CompleteCachedClassLoading(Improvement, &GC.getImprovementInfo(), "Civ4ImprovementInfos/ImprovementInfos/ImprovementInfo", &GC.getNumImprovementInfos(), "Terrain", "CIV4ImprovementInfos", &CvDLLUtilityIFaceBase::createImprovementInfoCacheObject);
    CompleteCachedClassLoading(Civic, &GC.getCivicInfo(), "Civ4CivicInfos/CivicInfos/CivicInfo", &GC.getNumCivicInfos(), "GameInfo", "CIV4CivicInfos", &CvDLLUtilityIFaceBase::createCivicInfoCacheObject);
    CompleteCachedClassLoading(Building, &GC.getBuildingInfo(), "Civ4BuildingInfos/BuildingInfos/BuildingInfo", &GC.getNumBuildingInfos(), "Buildings", "CIV4BuildingClassInfos", &CvDLLUtilityIFaceBase::createBuildingInfoCacheObject);
    CompleteCachedClassLoading(Bonus, &GC.getBonusInfo(), "Civ4BonusInfos/BonusInfos/BonusInfo", &GC.getNumBonusInfos(), "Terrain", "CIV4BonusInfos", &CvDLLUtilityIFaceBase::createBonusInfoCacheObject);
    CompleteCachedClassLoading(Handicap, &GC.getHandicapInfo(), "Civ4HandicapInfo/HandicapInfos/HandicapInfo", &GC.getNumHandicapInfos(), "GameInfo", "CIV4HandicapInfo", &CvDLLUtilityIFaceBase::createHandicapInfoCacheObject);
    CompleteCachedClassLoading(Tech, &GC.getTechInfo(), "Civ4TechInfos/TechInfos/TechInfo", &GC.getNumTechInfos(), "Technologies", "CIV4TechInfos", &CvDLLUtilityIFaceBase::createTechInfoCacheObject);
    CompleteCachedClassLoading(Unit, &GC.getUnitInfo(), "Civ4UnitInfos/UnitInfos/UnitInfo", &GC.getNumUnitInfos(), "Units", "CIV4UnitInfos", &CvDLLUtilityIFaceBase::createUnitInfoCacheObject);
    CompleteCachedClassLoading(LeaderHead, &GC.getLeaderHeadInfo(), "Civ4LeaderHeadInfos/LeaderHeadInfos/LeaderHeadInfo", &GC.getNumLeaderHeadInfos(), "Civilizations", "CIV4LeaderHeadInfos", &CvDLLUtilityIFaceBase::createLeaderHeadInfoCacheObject);
    CompleteCachedClassLoading(Civilization, &GC.getCivilizationInfo(), "Civ4CivilizationInfos/CivilizationInfos/CivilizationInfo", &GC.getNumCivilizationInfos(), "Civilizations", "CIV4CivilizationInfos", &CvDLLUtilityIFaceBase::createCivilizationInfoCacheObject);

    // Post Processing

	for (i=0;i<GC.getNumCivilizationInfos();i++)
	{
		// if the civilization is playable we will increment the playable var
		if (GC.getCivilizationInfo((CivilizationTypes) i).isPlayable())
		{
			GC.getNumPlayableCivilizationInfos() += 1;
		}

		// if the civilization is playable by AI increments num playable
		if (GC.getCivilizationInfo((CivilizationTypes) i).isAIPlayable())
		{
			GC.getNumAIPlayableCivilizationInfos() += 1;
		}
	}

	// Cant Modularly load Enum based data...

	bLoaded = LoadCivXml(m_pFXml, "Terrain/CIV4YieldInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Terrain/CIV4YieldInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		int iEnumVal = NUM_YIELD_TYPES;
		SetGlobalClassInfo(&GC.getYieldInfo(), "Civ4YieldInfos/YieldInfos/YieldInfo", &iEnumVal, true);
	}

	bLoaded = LoadCivXml(m_pFXml, "GameInfo/CIV4CommerceInfo.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for GameInfo/CIV4CommerceInfo.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		int iEnumVal = NUM_COMMERCE_TYPES;
		SetGlobalClassInfo(&GC.getCommerceInfo(), "Civ4CommerceInfo/CommerceInfos/CommerceInfo", &iEnumVal, true);
	}

	bLoaded = LoadCivXml(m_pFXml, "GameInfo/CIV4Hints.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for GameInfo/CIV4Hints.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalClassInfo(&GC.getHints(), "Civ4Hints/HintInfos/HintInfo", &GC.getNumHints());
	}

	bLoaded = LoadCivXml(m_pFXml, "GameInfo/CIV4GameOptionInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for GameInfo/CIV4GameOptionInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		int iEnumVal = NUM_GAMEOPTION_TYPES;
		SetGlobalClassInfo(&GC.getGameOptionInfo(), "Civ4GameOptionInfos/GameOptionInfos/GameOptionInfo", &iEnumVal, true);
	}

	bLoaded = LoadCivXml(m_pFXml, "GameInfo/CIV4MPOptionInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for GameInfo/CIV4MPOptionInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		int iEnumVal = NUM_MPOPTION_TYPES;
		SetGlobalClassInfo(&GC.getMPOptionInfo(), "Civ4MPOptionInfos/MPOptionInfos/MPOptionInfo", &iEnumVal, true);
	}

	bLoaded = LoadCivXml(m_pFXml, "GameInfo/CIV4ForceControlInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for GameInfo/CIV4ForceControlInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		int iEnumVal = NUM_FORCECONTROL_TYPES;
		SetGlobalClassInfo(&GC.getForceControlInfo(), "Civ4ForceControlInfos/ForceControlInfos/ForceControlInfo", &iEnumVal, true);
	}

	bLoaded = LoadCivXml(m_pFXml, "Interface/CIV4SlideShowInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Interface/CIV4SlideShowInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalClassInfo(&GC.getSlideShowInfo(), "Civ4SlideShowInfos/SlideShowInfos/SlideShowInfo", &GC.getNumSlideShowInfos());
		SetGlobalClassInfo(&GC.getSlideShowRandomInfo(), "Civ4SlideShowInfos/SlideShowRandomInfos/SlideShowRandomInfo", &GC.getNumSlideShowRandomInfos());
	}

	bLoaded = LoadCivXml(m_pFXml, "Interface/CIV4WorldPickerInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Interface/CIV4WorldPickerInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalClassInfo(&GC.getWorldPickerInfo(), "Civ4WorldPickerInfos/WorldPickerInfos/WorldPickerInfo", &GC.getNumWorldPickerInfos());
	}

	UpdateProgressCB("GlobalOther");

	DestroyFXml();

	return true;
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   LoadPostMenuGlobals()
//
//  PURPOSE :   loads global xml data which isn't needed for the main menus
//		this data is loaded as a secodn stage, when the game is launched
//
//------------------------------------------------------------------------------------------------------
bool CvXMLLoadUtility::LoadPostMenuGlobals()
{
	PROFILE_FUNC();
	if (!CreateFXml())
	{
		return false;
	}

	CvString *pszDefaultUnits = NULL;		// this will hold the default units for the unit classes
	bool bLoaded = false;	// used to make sure that the xml file was loaded correctly
	CvCacheObject* cache;

	//throne room disabled
	/*UpdateProgressCB("GlobalOther");

	// load the new FXml variable with the CIV4ThroneRoomInfos.xml file
	bLoaded = LoadCivXml(m_pFXml, "Interface/CIV4ThroneRoomInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Interface/CIV4ThroneRoomInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalClassInfo(&GC.getThroneRoomCamera(), "Civ4ThroneRoomInfos/ThroneRoomInfos/ThroneRoomCamera", &GC.getNumThroneRoomCameras());
		SetGlobalClassInfo(&GC.getThroneRoomInfo(), "Civ4ThroneRoomInfos/ThroneRoomInfos/ThroneRoomInfo", &GC.getNumThroneRoomInfos());
	}

	UpdateProgressCB("GlobalOther");

	// load the new FXml variable with the CIV4ThroneRoomStyleInfos.xml file
	bLoaded = LoadCivXml(m_pFXml, "Interface/CIV4ThroneRoomStyleInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Interface/CIV4ThroneRoomStyleInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalClassInfo(&GC.getThroneRoomStyleInfo(), "Civ4ThroneRoomStyleInfos/ThroneRoomStyleInfos/ThroneRoomStyleInfo", &GC.getNumThroneRoomStyleInfos());
	}
	*/

	UpdateProgressCB("GlobalOther");

	// load the new FXml variable with the CIV4RouteModelInfos.xml file
	bLoaded = LoadCivXml(m_pFXml, "Art/CIV4RouteModelInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Art/CIV4RouteModelInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalClassInfo(&GC.getRouteModelInfo(), "Civ4RouteModelInfos/RouteModelInfos/RouteModelInfo", &GC.getNumRouteModelInfos());
	}

	UpdateProgressCB("GlobalOther");

	// load the new FXml variable with the CIV4RiverInfos.xml file
	bLoaded = LoadCivXml(m_pFXml, "Misc/CIV4RiverInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Misc/CIV4RiverInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalRiverInfo(&GC.getRiverInfo(), "Civ4RiverInfos/RiverInfos/RiverInfo", &GC.getNumRiverInfos(), &GC.getRiverZBias());
	}

	// load the new FXml variable with the CIV4Buildings.xml file
	bLoaded = LoadCivXml(m_pFXml, "Art/CIV4RiverModelInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Art/CIV4RiverModelInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalClassInfo(&GC.getRiverModelInfo(), "Civ4RiverModelInfos/RiverModelInfos/RiverModelInfo", &GC.getNumRiverModelInfos());
	}

	// load the new FXml variable with the CIV4Buildings.xml file
	bLoaded = LoadCivXml(m_pFXml, "Misc/CIV4WaterPlaneInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Misc/CIV4WaterPlaneInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalClassInfo(&GC.getWaterPlaneInfo(), "Civ4WaterPlaneInfos/WaterPlaneInfos/WaterPlaneInfo", &GC.getNumWaterPlaneInfos());
	}

	UpdateProgressCB("GlobalOther");
	// load the new FXml variable with the CIV4ProcessInfo.xml file
	bLoaded = LoadCivXml(m_pFXml, "GameInfo/CIV4ProcessInfo.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for GameInfo/CIV4ProcessInfo.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalClassInfo(&GC.getProcessInfo(), "Civ4ProcessInfo/ProcessInfos/ProcessInfo", &GC.getNumProcessInfos());
	}

	UpdateProgressCB("GlobalOther");

	// load the new FXml variable with the CIV4Buildings.xml file
	bLoaded = LoadCivXml(m_pFXml, "GameInfo/CIV4EmphasizeInfo.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for GameInfo/CIV4EmphasizeInfo.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalClassInfo(&GC.getEmphasizeInfo(), "Civ4EmphasizeInfo/EmphasizeInfos/EmphasizeInfo", &GC.getNumEmphasizeInfos());
	}

	UpdateProgressCB("GlobalOther");

	bLoaded = LoadCivXml(m_pFXml, "Units/CIV4MissionInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Units/CIV4MissionInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		int iEnumVal = NUM_MISSION_TYPES;
		SetGlobalClassInfo(&GC.getMissionInfo(), "Civ4MissionInfos/MissionInfos/MissionInfo", &iEnumVal, true);
	}

	bLoaded = LoadCivXml(m_pFXml, "Units/CIV4ControlInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Units/CIV4ControlInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		int iEnumVal = NUM_CONTROL_TYPES;
		SetGlobalClassInfo(&GC.getControlInfo(), "Civ4ControlInfos/ControlInfos/ControlInfo", &iEnumVal, true);
	}

	bLoaded = LoadCivXml(m_pFXml, "Units/CIV4CommandInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Units/CIV4CommandInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded)
	{
		if (Validate())
		{
			int iEnumVal = NUM_COMMAND_TYPES;
			SetGlobalClassInfo(&GC.getCommandInfo(), "Civ4CommandInfos/CommandInfos/CommandInfo", &iEnumVal, true);
		}
	}

	bLoaded = LoadCivXml(m_pFXml, "Units/CIV4AutomateInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Units/CIV4AutomateInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalClassInfo(&GC.getAutomateInfo(), "Civ4AutomateInfos/AutomateInfos/AutomateInfo", &GC.getNumAutomateInfos());
	}

	UpdateProgressCB("GlobalOther");

	// load the new FXml variable with the CIV4VoteInfo.xml file
	bLoaded = LoadCivXml(m_pFXml, "GameInfo/CIV4VoteInfo.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Units/CIV4VoteInfo.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalClassInfo(&GC.getVoteInfo(), "Civ4VoteInfo/VoteInfos/VoteInfo", &GC.getNumVoteInfos());
	}

	UpdateProgressCB("GlobalOther");

	bLoaded = LoadCivXml(m_pFXml, "Interface/CIV4CameraInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Interface/CIV4CameraInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalClassInfo(&GC.getCameraInfo(), "Civ4CameraInfos/CameraInfos/CameraInfo", &GC.getNumCameraInfos());
	}

	bLoaded = LoadCivXml(m_pFXml, "Interface/CIV4InterfaceModeInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Interface/CIV4InterfaceModeInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded)
	{
		if (Validate())
		{
			int iEnumVal = NUM_INTERFACEMODE_TYPES;
			SetGlobalClassInfo(&GC.getInterfaceModeInfo(), "Civ4InterfaceModeInfos/InterfaceModeInfos/InterfaceModeInfo", &iEnumVal, true);
		}
	}

	if (bLoaded && Validate())
	{
		SetGlobalActionInfo(&GC.getActionInfo(), &GC.getNumActionInfos());
	}


	// Load the formation info
	bLoaded = LoadCivXml(m_pFXml, "Units/CIV4FormationInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Units/CIV4FormationInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded)
	{
		if (Validate())
		{
			SetGlobalClassInfo(&GC.getUnitFormationInfo(), "UnitFormations/UnitFormation", &GC.getNumUnitFormationInfos());
		}
	}

	// Load the attachable infos
	bLoaded = LoadCivXml(m_pFXml, "Misc/CIV4AttachableInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Misc/CIV4AttachableInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalClassInfo(&GC.getAttachableInfo(), "Civ4AttachableInfos/AttachableInfos/AttachableInfo", &GC.getNumAttachableInfos());
	}

	cache = gDLL->createDiplomacyInfoCacheObject("CIV4DiplomacyInfos.dat");		// cache file name
	if (!gDLL->cacheRead(cache, "xml\\GameInfo\\CIV4DiplomacyInfos.xml"))	// src data file name
	{
		// load the new FXml variable with the CIV4Buildings.xml file
		bLoaded = LoadCivXml(m_pFXml, "GameInfo/CIV4DiplomacyInfos.xml");
		if (!bLoaded)
		{
			char	szMessage[1024];
			sprintf( szMessage, "LoadXML call failed for GameInfo/CIV4DiplomacyInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
			gDLL->MessageBox(szMessage, "XML Load Error");
		}
		if (bLoaded)
		{
			if (Validate())
			{
				SetGlobalClassInfo(&GC.getDiplomacyInfo(), "Civ4DiplomacyInfos/DiplomacyInfos/DiplomacyInfo", &GC.getNumDiplomacyInfos());

				// write civ info to cache
				bool bOk = gDLL->cacheWrite(cache);
				if (!bOk)
				{
					char	szMessage[1024];
					sprintf( szMessage, "Failed writing to Diplomacy cache. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
					gDLL->MessageBox(szMessage, "XML Caching Error");
				}
				if (bOk)
				{
					logMsg("Wrote Civ4DiplomacyInfos to cache");
				}
			}
		}
	}
	else
	{
		logMsg("Read CIV4DiplomacyInfos from cache");
		bLoaded	= true;
	}
	gDLL->destroyCache(cache);

	// load the new FXml variable with the Civ4QuestInfos.xml file
	bLoaded = LoadCivXml(m_pFXml, "Misc/Civ4QuestInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Misc/Civ4QuestInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalClassInfo(&GC.getQuestInfo(), "Civ4QuestInfos/QuestInfo", &GC.getNumQuestInfos());
	}

	// load the new FXml variable with the Civ4TutorialInfos.xml file
	bLoaded = LoadCivXml(m_pFXml, "Misc/Civ4TutorialInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for Misc/Civ4TutorialInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Load Error");
	}
	if (bLoaded && Validate())
	{
		SetGlobalClassInfo(&GC.getTutorialInfo(), "Civ4TutorialInfos/TutorialInfo", &GC.getNumTutorialInfos());
	}

	SAFE_DELETE_ARRAY(pszDefaultUnits);

	DestroyFXml();
	return true;
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetGlobalStringArray(TCHAR (**ppszString)[256], char* szTagName, int* iNumVals)
//
//  PURPOSE :   takes the szTagName parameter and if it finds it in the m_pFXml member variable
//				then it loads the ppszString parameter with the string values under it and the
//				iNumVals with the total number of tags with the szTagName in the xml file
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetGlobalStringArray(CvString **ppszString, char* szTagName, int* iNumVals, bool bUseEnum)
{
	PROFILE_FUNC();
	logMsg("SetGlobalStringArray %s\n", szTagName);

	int i=0;					//loop counter
	CvString *pszString;	// hold the local pointer to the newly allocated string memory
	pszString = NULL;			// null out the local string pointer so that it can be checked at the
	// end of the function in an FAssert

	// if we locate the szTagName, the current node is set to the first instance of the tag name in the xml file
	if (gDLL->getXMLIFace()->LocateNode(m_pFXml,szTagName))
	{
		if (!bUseEnum)
		{
			// get the total number of times this tag appears in the xml
			*iNumVals = gDLL->getXMLIFace()->NumOfElementsByTagName(m_pFXml,szTagName);
		}
		// initialize the memory based on the total number of tags in the xml and the 256 character length we selected
		*ppszString = new CvString[*iNumVals];
		// set the local pointer to the memory just allocated
		pszString = *ppszString;

		// loop through each of the tags
		for (i=0;i<*iNumVals;i++)
		{
			// get the string value at the current node
			GetXmlVal(pszString[i]);
			GC.setTypesEnum(pszString[i], i);

			// if can't set the current node to a sibling node we will break out of the for loop
			// otherwise we will keep looping
			if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
			{
				break;
			}
		}
	}

	// if the local string pointer is null then we weren't able to find the szTagName in the xml
	// so we will FAssert to let whoever know it
	if (!pszString)
	{
		char	szMessage[1024];
		sprintf( szMessage, "Error locating tag node in SetGlobalStringArray function \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetGlobalRouteInfo(CvRouteInfo **ppRouteInfos, char* szTagName, int* iNumVals)
//
//  PURPOSE :   Takes the szTagName parameter and if it exists in the xml it loads the ppRouteInfos
//				with the value under it and sets the value of the iNumVals parameter to the total number
//				of occurances of the szTagName tag in the xml.
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetGlobalRouteInfo(CvRouteInfo **ppRouteInfos, char* szTagName, int* iNumVals, float *fZbias)
{
	PROFILE_FUNC();
	logMsg("SetGlobalRouteInfo %s\n", szTagName);
	int i=0;				//loop counter
	CvRouteInfo* pRouteInfos;	// local pointer to the route info memory

	// null the local pointer so it can be checked at the end of the function
	pRouteInfos = NULL;

	// if we successfully locate the szTagName node
	if (gDLL->getXMLIFace()->LocateNode(m_pFXml,szTagName))
	{
		// get the number of times the szTagName tag appears in the xml file
		*iNumVals = gDLL->getXMLIFace()->NumOfElementsByTagName(m_pFXml,szTagName);
		// allocate memory for the route infos based on the number of tags in the xml
		*ppRouteInfos = new CvRouteInfo[*iNumVals];
		// set the local pointer to the memory we just allocated
		pRouteInfos = *ppRouteInfos;
		// set the current xml node to it's parent node since we are on the first child of the root tag group
		gDLL->getXMLIFace()->SetToParent(m_pFXml);
		GetChildXmlValByName(fZbias,  "fZBias");
		gDLL->getXMLIFace()->LocateNode(m_pFXml,szTagName);

		// loop through each tag
		for (i=0;i<*iNumVals;i++)
		{
			SkipToNextVal();	// skip to the next non-comment node

			if (!pRouteInfos[i].read(this))
				break;

			GC.setInfoTypeFromString(pRouteInfos[i].getType(), i);	// add type to global info type hash map
			if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
			{
				break;
			}
		}
	}

	// if we didn't find the tag name in the xml then we never set the local pointer to the
	// newly allocated memory and there for we will FAssert to let people know this most
	// interesting fact
	if(!pRouteInfos)
	{
		char	szMessage[1024];
		sprintf( szMessage, "Error finding tag node in SetGlobalRouteInfo function \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
}


//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetGlobalRiverInfo(CvRiverInfo **ppRiverInfos, char* szTagName, int* iNumVals)
//
//  PURPOSE :   Takes the szTagName parameter and if it exists in the xml it loads the ppRiverInfos
//				with the value under it and sets the value of the iNumVals parameter to the total number
//				of occurances of the szTagName tag in the xml.
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetGlobalRiverInfo(CvRiverInfo **ppRiverInfos, char* szTagName, int* iNumVals,float *fZBias)
{
	PROFILE_FUNC();
	logMsg("SetGlobalRiverInfo %s\n", szTagName);
	int i=0;							// loop counter
	CvRiverInfo* pRiverInfos;	// local pointer to the River info memory

	// null the local pointer so it can be checked at the end of the function
	pRiverInfos = NULL;

	// if we successfully locate the szTagName node
	if (gDLL->getXMLIFace()->LocateNode(m_pFXml,szTagName))
	{
		// get the number of times the szTagName tag appears in the xml file
		*iNumVals = gDLL->getXMLIFace()->NumOfElementsByTagName(m_pFXml,szTagName);
		// allocate memory for the River infos based on the number of tags in the xml
		*ppRiverInfos = new CvRiverInfo[*iNumVals];
		// set the local pointer to the memory we just allocated
		pRiverInfos = *ppRiverInfos;
		// set the current xml node to it's parent node since we are on the first child of the root tag group
		gDLL->getXMLIFace()->SetToParent(m_pFXml);
		GetChildXmlValByName(fZBias,  "fZBias");
		gDLL->getXMLIFace()->LocateNode(m_pFXml,szTagName);

		// loop through each tag
		for (i=0;i<*iNumVals;i++)
		{
			SkipToNextVal();	// skip to the next non-comment node

			if (!pRiverInfos[i].read(this))
				break;

			GC.setInfoTypeFromString(pRiverInfos[i].getType(), i);	// add type to global info type hash map
			if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
			{
				break;
			}
		}
	}

	// if we didn't find the tag name in the xml then we never set the local pointer to the
	// newly allocated memory and there for we will FAssert to let people know this most
	// interesting fact
	if(!pRiverInfos)
	{
		char	szMessage[1024];
		sprintf( szMessage, "Error finding tag node in SetGlobalRiverInfo function \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetGlobalActionInfo(CvActionInfo** ppActionInfo, int* iNumVals)
//
//  PURPOSE :   Takes the szTagName parameter and if it exists in the xml it loads the ppActionInfo
//				with the value under it and sets the value of the iNumVals parameter to the total number
//				of occurances of the szTagName tag in the xml.
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetGlobalActionInfo(CvActionInfo** ppActionInfo, int* iNumVals)
{
	PROFILE_FUNC();
	logMsg("SetGlobalActionInfo\n");
	int i=0;					//loop counter
	int j=0;					//loop counter
	CvActionInfo* pActionInfo;	// local pointer to the action info memory

	// null out the local pointer so that it can be checked at the end of this function
	pActionInfo = NULL;

	*iNumVals = 0;
	if(!(NUM_INTERFACEMODE_TYPES > 0))
	{
		char	szMessage[1024];
		sprintf( szMessage, "NUM_INTERFACE_TYPES is not greater than zero in CvXMLLoadUtility::SetGlobalActionInfo \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	if(!(GC.getNumBuildInfos() > 0))
	{
		char	szMessage[1024];
		sprintf( szMessage, "GC.getNumBuildInfos() is not greater than zero in CvXMLLoadUtility::SetGlobalActionInfo \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	if(!(GC.getNumPromotionInfos() > 0))
	{
		char	szMessage[1024];
		sprintf( szMessage, "GC.getNumPromotionInfos() is not greater than zero in CvXMLLoadUtility::SetGlobalActionInfo \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	if(!(GC.getNumReligionInfos() > 0) )
	{
		char	szMessage[1024];
		sprintf( szMessage, "GC.getNumReligionInfos() is not greater than zero in CvXMLLoadUtility::SetGlobalActionInfo \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	if(!(GC.getNumUnitClassInfos() > 0) )
	{
		char	szMessage[1024];
		sprintf( szMessage, "GC.getNumUnitClassInfos() is not greater than zero in CvXMLLoadUtility::SetGlobalActionInfo \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	if(!(GC.getNumSpecialistInfos() > 0) )
	{
		char	szMessage[1024];
		sprintf( szMessage, "GC.getNumSpecialistInfos() is not greater than zero in CvXMLLoadUtility::SetGlobalActionInfo \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	if(!(GC.getNumBuildingInfos() > 0) )
	{
		char	szMessage[1024];
		sprintf( szMessage, "GC.getNumBuildingInfos() is not greater than zero in CvXMLLoadUtility::SetGlobalActionInfo \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	if(!(NUM_CONTROL_TYPES > 0) )
	{
		char	szMessage[1024];
		sprintf( szMessage, "NUM_CONTROL_TYPES is not greater than zero in CvXMLLoadUtility::SetGlobalActionInfo \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	if(!(GC.getNumAutomateInfos() > 0) )
	{
		char	szMessage[1024];
		sprintf( szMessage, "GC.getNumAutomateInfos() is not greater than zero in CvXMLLoadUtility::SetGlobalActionInfo \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	if(!(NUM_COMMAND_TYPES > 0) )
	{
		char	szMessage[1024];
		sprintf( szMessage, "NUM_COMMAND_TYPES is not greater than zero in CvXMLLoadUtility::SetGlobalActionInfo \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	if(!(NUM_MISSION_TYPES > 0) )
	{
		char	szMessage[1024];
		sprintf( szMessage, "NUM_MISSION_TYPES is not greater than zero in CvXMLLoadUtility::SetGlobalActionInfo \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	// allocate memory for the action info based on the number above
	*ppActionInfo = new CvActionInfo[	(*iNumVals) +
		NUM_INTERFACEMODE_TYPES +
		GC.getNumBuildInfos() +
		GC.getNumPromotionInfos() +
		GC.getNumReligionInfos() +
		GC.getNumUnitInfos() +
		GC.getNumSpecialistInfos() +
		GC.getNumBuildingInfos() +
		NUM_CONTROL_TYPES +
		NUM_COMMAND_TYPES +
		GC.getNumAutomateInfos() +
		NUM_MISSION_TYPES];
	// set the local pointer to the memory we just allocated
	pActionInfo = *ppActionInfo;

	int* piOrderedIndex=NULL;

	int iNumActionInfos =(*iNumVals) +
		NUM_INTERFACEMODE_TYPES +
		GC.getNumBuildInfos() +
		GC.getNumPromotionInfos() +
		GC.getNumReligionInfos() +
		GC.getNumUnitInfos() +
		GC.getNumSpecialistInfos() +
		GC.getNumBuildingInfos() +
		NUM_CONTROL_TYPES +
		NUM_COMMAND_TYPES +
		GC.getNumAutomateInfos() +
		NUM_MISSION_TYPES;
	int* piIndexList = new int[iNumActionInfos];
	int* piPriorityList = new int[iNumActionInfos];
	int* piActionInfoTypeList = new int[iNumActionInfos];

	int iTotalActionInfoCount=0;
	// loop through control info
	for (i=0;i<NUM_COMMAND_TYPES;i++)
	{
		piIndexList[iTotalActionInfoCount] = i;
		piPriorityList[iTotalActionInfoCount] = GC.getCommandInfo((CommandTypes)i).getOrderPriority();
		piActionInfoTypeList[iTotalActionInfoCount] = ACTIONSUBTYPE_COMMAND;
		iTotalActionInfoCount++;
	}

	for (i=0;i<NUM_INTERFACEMODE_TYPES;i++)
	{
		piIndexList[iTotalActionInfoCount] = i;
		piPriorityList[iTotalActionInfoCount] = GC.getInterfaceModeInfo((InterfaceModeTypes)i).getOrderPriority();
		piActionInfoTypeList[iTotalActionInfoCount] = ACTIONSUBTYPE_INTERFACEMODE;
		iTotalActionInfoCount++;
	}

	for (i=0;i<GC.getNumBuildInfos();i++)
	{
		piIndexList[iTotalActionInfoCount] = i;
		piPriorityList[iTotalActionInfoCount] = GC.getBuildInfo((BuildTypes)i).getOrderPriority();
		piActionInfoTypeList[iTotalActionInfoCount] = ACTIONSUBTYPE_BUILD;
		iTotalActionInfoCount++;
	}

	for (i=0;i<GC.getNumPromotionInfos();i++)
	{
		piIndexList[iTotalActionInfoCount] = i;
		piPriorityList[iTotalActionInfoCount] = GC.getPromotionInfo((PromotionTypes)i).getOrderPriority();
		piActionInfoTypeList[iTotalActionInfoCount] = ACTIONSUBTYPE_PROMOTION;
		iTotalActionInfoCount++;
	}

	for (i=0;i<GC.getNumUnitInfos();i++)
	{
		piIndexList[iTotalActionInfoCount] = i;
		piPriorityList[iTotalActionInfoCount] = GC.getUnitInfo((UnitTypes)i).getOrderPriority();
		piActionInfoTypeList[iTotalActionInfoCount] = ACTIONSUBTYPE_UNIT;
		iTotalActionInfoCount++;
	}

	for (i=0;i<GC.getNumReligionInfos();i++)
	{
		piIndexList[iTotalActionInfoCount] = i;
		piPriorityList[iTotalActionInfoCount] = GC.getReligionInfo((ReligionTypes)i).getOrderPriority();
		piActionInfoTypeList[iTotalActionInfoCount] = ACTIONSUBTYPE_RELIGION;
		iTotalActionInfoCount++;
	}

	for (i=0;i<GC.getNumSpecialistInfos();i++)
	{
		piIndexList[iTotalActionInfoCount] = i;
		piPriorityList[iTotalActionInfoCount] = GC.getSpecialistInfo((SpecialistTypes)i).getOrderPriority();
		piActionInfoTypeList[iTotalActionInfoCount] = ACTIONSUBTYPE_SPECIALIST;
		iTotalActionInfoCount++;
	}

	for (i=0;i<GC.getNumBuildingInfos();i++)
	{
		piIndexList[iTotalActionInfoCount] = i;
		piPriorityList[iTotalActionInfoCount] = GC.getBuildingInfo((BuildingTypes)i).getOrderPriority();
		piActionInfoTypeList[iTotalActionInfoCount] = ACTIONSUBTYPE_BUILDING;
		iTotalActionInfoCount++;
	}

	for (i=0;i<NUM_CONTROL_TYPES;i++)
	{
		piIndexList[iTotalActionInfoCount] = i;
		piPriorityList[iTotalActionInfoCount] = GC.getControlInfo((ControlTypes)i).getOrderPriority();
		piActionInfoTypeList[iTotalActionInfoCount] = ACTIONSUBTYPE_CONTROL;
		iTotalActionInfoCount++;
	}

	for (i=0;i<GC.getNumAutomateInfos();i++)
	{
		piIndexList[iTotalActionInfoCount] = i;
		piPriorityList[iTotalActionInfoCount] = GC.getAutomateInfo(i).getOrderPriority();
		piActionInfoTypeList[iTotalActionInfoCount] = ACTIONSUBTYPE_AUTOMATE;
		iTotalActionInfoCount++;
	}

	for (i=0;i<NUM_MISSION_TYPES;i++)
	{
		piIndexList[iTotalActionInfoCount] = i;
		piPriorityList[iTotalActionInfoCount] = GC.getMissionInfo((MissionTypes)i).getOrderPriority();
		piActionInfoTypeList[iTotalActionInfoCount] = ACTIONSUBTYPE_MISSION;
		iTotalActionInfoCount++;
	}

	SAFE_DELETE_ARRAY(piOrderedIndex);
	piOrderedIndex = new int[iNumActionInfos];

	orderHotkeyInfo(&piOrderedIndex, piPriorityList, iNumActionInfos);
	for (i=0;i<iNumActionInfos;i++)
	{
		pActionInfo[i + *iNumVals].setOriginalIndex(piIndexList[piOrderedIndex[i]]);
		pActionInfo[i + *iNumVals].setSubType((ActionSubTypes)piActionInfoTypeList[piOrderedIndex[i]]);
		if ((ActionSubTypes)piActionInfoTypeList[piOrderedIndex[i]] == ACTIONSUBTYPE_COMMAND)
		{
			GC.getCommandInfo((CommandTypes)piIndexList[piOrderedIndex[i]]).setActionInfoIndex(i);
		}
		else if ((ActionSubTypes)piActionInfoTypeList[piOrderedIndex[i]] == ACTIONSUBTYPE_INTERFACEMODE)
		{
			GC.getInterfaceModeInfo((InterfaceModeTypes)piIndexList[piOrderedIndex[i]]).setActionInfoIndex(i);
		}
		else if ((ActionSubTypes)piActionInfoTypeList[piOrderedIndex[i]] == ACTIONSUBTYPE_BUILD)
		{
			GC.getBuildInfo((BuildTypes)piIndexList[piOrderedIndex[i]]).setMissionType(FindInInfoClass("MISSION_BUILD", GC.getMissionInfo(), sizeof(GC.getMissionInfo((MissionTypes) 0)), NUM_MISSION_TYPES));
			GC.getBuildInfo((BuildTypes)piIndexList[piOrderedIndex[i]]).setActionInfoIndex(i);
		}
		else if ((ActionSubTypes)piActionInfoTypeList[piOrderedIndex[i]] == ACTIONSUBTYPE_PROMOTION)
		{
			GC.getPromotionInfo((PromotionTypes)piIndexList[piOrderedIndex[i]]).setCommandType(FindInInfoClass("COMMAND_PROMOTION", GC.getCommandInfo(), sizeof(GC.getCommandInfo((CommandTypes) 0)), NUM_COMMAND_TYPES));
			GC.getPromotionInfo((PromotionTypes)piIndexList[piOrderedIndex[i]]).setActionInfoIndex(i);
			GC.getPromotionInfo((PromotionTypes)piIndexList[piOrderedIndex[i]]).setHotKeyDescription(GC.getPromotionInfo((PromotionTypes)piIndexList[piOrderedIndex[i]]).getTextKeyWide(), GC.getCommandInfo((CommandTypes)(GC.getPromotionInfo((PromotionTypes)piIndexList[piOrderedIndex[i]]).getCommandType())).getTextKeyWide(), CreateHotKeyFromDescription(GC.getPromotionInfo((PromotionTypes)piIndexList[piOrderedIndex[i]]).getHotKey(), GC.getPromotionInfo((PromotionTypes)piIndexList[piOrderedIndex[i]]).isShiftDown(), GC.getPromotionInfo((PromotionTypes)piIndexList[piOrderedIndex[i]]).isAltDown(), GC.getPromotionInfo((PromotionTypes)piIndexList[piOrderedIndex[i]]).isCtrlDown()));
		}
		else if ((ActionSubTypes)piActionInfoTypeList[piOrderedIndex[i]] == ACTIONSUBTYPE_UNIT)
		{
			GC.getUnitInfo((UnitTypes)piIndexList[piOrderedIndex[i]]).setCommandType(FindInInfoClass("COMMAND_UPGRADE", GC.getCommandInfo(), sizeof(GC.getCommandInfo((CommandTypes) 0)), NUM_COMMAND_TYPES));
			GC.getUnitInfo((UnitTypes)piIndexList[piOrderedIndex[i]]).setActionInfoIndex(i);
			GC.getUnitInfo((UnitTypes)piIndexList[piOrderedIndex[i]]).setHotKeyDescription(GC.getUnitInfo((UnitTypes)piIndexList[piOrderedIndex[i]]).getTextKeyWide(), GC.getCommandInfo((CommandTypes)(GC.getUnitInfo((UnitTypes)piIndexList[piOrderedIndex[i]]).getCommandType())).getTextKeyWide(), CreateHotKeyFromDescription(GC.getUnitInfo((UnitTypes)piIndexList[piOrderedIndex[i]]).getHotKey(), GC.getUnitInfo((UnitTypes)piIndexList[piOrderedIndex[i]]).isShiftDown(), GC.getUnitInfo((UnitTypes)piIndexList[piOrderedIndex[i]]).isAltDown(), GC.getUnitInfo((UnitTypes)piIndexList[piOrderedIndex[i]]).isCtrlDown()));
		}
		else if ((ActionSubTypes)piActionInfoTypeList[piOrderedIndex[i]] == ACTIONSUBTYPE_RELIGION)
		{
			GC.getReligionInfo((ReligionTypes)piIndexList[piOrderedIndex[i]]).setMissionType(FindInInfoClass("MISSION_SPREAD", GC.getMissionInfo(), sizeof(GC.getMissionInfo((MissionTypes) 0)), NUM_MISSION_TYPES));
			GC.getReligionInfo((ReligionTypes)piIndexList[piOrderedIndex[i]]).setActionInfoIndex(i);
			GC.getReligionInfo((ReligionTypes)piIndexList[piOrderedIndex[i]]).setHotKeyDescription(GC.getReligionInfo((ReligionTypes)piIndexList[piOrderedIndex[i]]).getTextKeyWide(), GC.getMissionInfo((MissionTypes)(GC.getReligionInfo((ReligionTypes)piIndexList[piOrderedIndex[i]]).getMissionType())).getTextKeyWide(), CreateHotKeyFromDescription(GC.getReligionInfo((ReligionTypes)piIndexList[piOrderedIndex[i]]).getHotKey(), GC.getReligionInfo((ReligionTypes)piIndexList[piOrderedIndex[i]]).isShiftDown(), GC.getReligionInfo((ReligionTypes)piIndexList[piOrderedIndex[i]]).isAltDown(), GC.getReligionInfo((ReligionTypes)piIndexList[piOrderedIndex[i]]).isCtrlDown()));
		}
		else if ((ActionSubTypes)piActionInfoTypeList[piOrderedIndex[i]] == ACTIONSUBTYPE_SPECIALIST)
		{
			GC.getSpecialistInfo((SpecialistTypes)piIndexList[piOrderedIndex[i]]).setMissionType(FindInInfoClass("MISSION_JOIN", GC.getMissionInfo(), sizeof(GC.getMissionInfo((MissionTypes) 0)), NUM_MISSION_TYPES));
			GC.getSpecialistInfo((SpecialistTypes)piIndexList[piOrderedIndex[i]]).setActionInfoIndex(i);
			GC.getSpecialistInfo((SpecialistTypes)piIndexList[piOrderedIndex[i]]).setHotKeyDescription(GC.getSpecialistInfo((SpecialistTypes)piIndexList[piOrderedIndex[i]]).getTextKeyWide(), GC.getMissionInfo((MissionTypes)(GC.getSpecialistInfo((SpecialistTypes)piIndexList[piOrderedIndex[i]]).getMissionType())).getTextKeyWide(), CreateHotKeyFromDescription(GC.getSpecialistInfo((SpecialistTypes)piIndexList[piOrderedIndex[i]]).getHotKey(), GC.getSpecialistInfo((SpecialistTypes)piIndexList[piOrderedIndex[i]]).isShiftDown(), GC.getSpecialistInfo((SpecialistTypes)piIndexList[piOrderedIndex[i]]).isAltDown(), GC.getSpecialistInfo((SpecialistTypes)piIndexList[piOrderedIndex[i]]).isCtrlDown()));
		}
		else if ((ActionSubTypes)piActionInfoTypeList[piOrderedIndex[i]] == ACTIONSUBTYPE_BUILDING)
		{
			GC.getBuildingInfo((BuildingTypes)piIndexList[piOrderedIndex[i]]).setMissionType(FindInInfoClass("MISSION_CONSTRUCT", GC.getMissionInfo(), sizeof(GC.getMissionInfo((MissionTypes) 0)), NUM_MISSION_TYPES));
			GC.getBuildingInfo((BuildingTypes)piIndexList[piOrderedIndex[i]]).setActionInfoIndex(i);
			GC.getBuildingInfo((BuildingTypes)piIndexList[piOrderedIndex[i]]).setHotKeyDescription(GC.getBuildingInfo((BuildingTypes)piIndexList[piOrderedIndex[i]]).getTextKeyWide(), GC.getMissionInfo((MissionTypes)(GC.getBuildingInfo((BuildingTypes)piIndexList[piOrderedIndex[i]]).getMissionType())).getTextKeyWide(), CreateHotKeyFromDescription(GC.getBuildingInfo((BuildingTypes)piIndexList[piOrderedIndex[i]]).getHotKey(), GC.getBuildingInfo((BuildingTypes)piIndexList[piOrderedIndex[i]]).isShiftDown(), GC.getBuildingInfo((BuildingTypes)piIndexList[piOrderedIndex[i]]).isAltDown(), GC.getBuildingInfo((BuildingTypes)piIndexList[piOrderedIndex[i]]).isCtrlDown()));
		}
		else if ((ActionSubTypes)piActionInfoTypeList[piOrderedIndex[i]] == ACTIONSUBTYPE_CONTROL)
		{
			GC.getControlInfo((ControlTypes)piIndexList[piOrderedIndex[i]]).setActionInfoIndex(i);
		}
		else if ((ActionSubTypes)piActionInfoTypeList[piOrderedIndex[i]] == ACTIONSUBTYPE_AUTOMATE)
		{
			GC.getAutomateInfo(piIndexList[piOrderedIndex[i]]).setActionInfoIndex(i);
		}
		else if ((ActionSubTypes)piActionInfoTypeList[piOrderedIndex[i]] == ACTIONSUBTYPE_MISSION)
		{
			GC.getMissionInfo((MissionTypes)piIndexList[piOrderedIndex[i]]).setActionInfoIndex(i + *iNumVals);
		}
	}

	*iNumVals = iNumActionInfos;

	SAFE_DELETE_ARRAY(piOrderedIndex);
	SAFE_DELETE_ARRAY(piIndexList);
	SAFE_DELETE_ARRAY(piPriorityList);
	SAFE_DELETE_ARRAY(piActionInfoTypeList);
}


//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetGlobalAnimationPathInfo(CvAnimationPathInfo** ppAnimationPathInfo, char* szTagName, int* iNumVals)
//
//  PURPOSE :   Takes the szTagName parameter and if it exists in the xml it loads the ppAnimationPathInfo
//				with the value under it and sets the value of the iNumVals parameter to the total number
//				of occurances of the szTagName tag in the xml.
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetGlobalAnimationPathInfo(CvAnimationPathInfo** ppAnimationPathInfo, char* szTagName, int* iNumVals)
{
	PROFILE_FUNC();
	logMsg( "SetGlobalAnimationPathInfo %s\n", szTagName );

	int		i;						// Loop counters
	CvAnimationPathInfo * pAnimPathInfo;	// local pointer to the domain info memory

	if ( gDLL->getXMLIFace()->LocateNode(m_pFXml, szTagName ))
	{
		// get the number of times the szTagName tag appears in the xml file
		*iNumVals = gDLL->getXMLIFace()->NumOfElementsByTagName(m_pFXml,szTagName);

		// allocate memory for the domain info based on the number above
		*ppAnimationPathInfo = new CvAnimationPathInfo[*iNumVals];
		pAnimPathInfo = *ppAnimationPathInfo;

		gDLL->getXMLIFace()->SetToParent(m_pFXml);
		gDLL->getXMLIFace()->SetToChild(m_pFXml);
		gDLL->getXMLIFace()->SetToChild(m_pFXml);


		// Loop through each tag.
		for (i=0;i<*iNumVals;i++)
		{
			SkipToNextVal();	// skip to the next non-comment node

			if (!pAnimPathInfo[i].read(this))
				break;
			GC.setInfoTypeFromString(pAnimPathInfo[i].getType(), i);	// add type to global info type hash map
			if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
			{
				break;
			}
		}
	}

	// if we didn't find the tag name in the xml then we never set the local pointer to the
	// newly allocated memory and there for we will FAssert to let people know this most
	// interesting fact
	if(!pAnimPathInfo )
	{
		char	szMessage[1024];
		sprintf( szMessage, "Error finding tag node in SetGlobalAnimationPathInfo function \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
}


//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetGlobalUnitScales(float* pfLargeScale, float* pfSmallScale, char* szTagName)
//
//  PURPOSE :   Takes the szTagName parameter and if it exists in the xml it loads the ppPromotionInfo
//				with the value under it and sets the value of the iNumVals parameter to the total number
//				of occurances of the szTagName tag in the xml.
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetGlobalUnitScales(float* pfLargeScale, float* pfSmallScale, char* szTagName)
{
	PROFILE_FUNC();
	logMsg("SetGlobalUnitScales %s\n", szTagName);
	// if we successfully locate the szTagName node
	if (gDLL->getXMLIFace()->LocateNode(m_pFXml,szTagName))
	{
		// call the function that sets the FXml pointer to the first non-comment child of
		// the current tag and gets the value of that new node
		if (GetChildXmlVal(pfLargeScale))
		{
			// set the current xml node to it's next sibling and then
			// get the sibling's TCHAR value
			GetNextXmlVal(pfSmallScale);

			// set the current xml node to it's parent node
			gDLL->getXMLIFace()->SetToParent(m_pFXml);
		}
	}
	else
	{
		// if we didn't find the tag name in the xml then we never set the local pointer to the
		// newly allocated memory and there for we will FAssert to let people know this most
		// interesting fact
		char	szMessage[1024];
		sprintf( szMessage, "Error finding tag node in SetGlobalUnitScales function \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
}


//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetGameText()
//
//  PURPOSE :   Reads game text info from XML and adds it to the translation manager
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetGameText(const char* szTextGroup, const char* szTagName)
{
	PROFILE_FUNC();
	logMsg("SetGameText %s\n", szTagName);
	int i=0;		//loop counter - Index into pTextInfo

	if (gDLL->getXMLIFace()->LocateNode(m_pFXml, szTextGroup)) // Get the Text Group 1st
	{
		int iNumVals = gDLL->getXMLIFace()->GetNumChildren(m_pFXml);	// Get the number of Children that the Text Group has
		gDLL->getXMLIFace()->LocateNode(m_pFXml, szTagName); // Now switch to the TEXT Tag
		gDLL->getXMLIFace()->SetToParent(m_pFXml);
		gDLL->getXMLIFace()->SetToChild(m_pFXml);

		// loop through each tag
		for (i=0; i < iNumVals; i++)
		{
			CvGameText textInfo;
			textInfo.read(this);

			gDLL->addText(textInfo.getType() /*id*/, textInfo.getText(), textInfo.getGender(), textInfo.getPlural());
			if (!gDLL->getXMLIFace()->NextSibling(m_pFXml) && i!=iNumVals-1)
			{
				char	szMessage[1024];
				sprintf( szMessage, "failed to find sibling \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
				gDLL->MessageBox(szMessage, "XML Error");
				break;
			}
		}
	}
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetGlobalClassInfo - This is a template function that is USED FOR ALMOST ALL INFO CLASSES.
//		Each info class should have a read(CvXMLLoadUtility*) function that is responsible for initializing
//		the class from xml data.
//
//  PURPOSE :   takes the szTagName parameter and loads the ppszString with the text values
//				under the tags.  This will be the hints displayed during game initialization and load
//
//------------------------------------------------------------------------------------------------------
template <class T>
void CvXMLLoadUtility::SetGlobalClassInfo(T **ppClassInfos, char* szTagName, int* iNumVals, bool bUseEnum)
{
	char szLog[256];
	sprintf(szLog, "SetGlobalClassInfo (%s)", szTagName);
	PROFILE(szLog);
	logMsg(szLog);

	int i=0;					//loop counter
	T* pClassInfo = NULL;	// local pointer to the building info memory

	// if we successfully locate the tag name in the xml file
	if (gDLL->getXMLIFace()->LocateNode(m_pFXml,szTagName))
	{
		if (!bUseEnum)
		{
			// get the number of times the szTagName tag appears in the xml file
			*iNumVals = gDLL->getXMLIFace()->NumOfElementsByTagName(m_pFXml,szTagName);
		}

		// allocate memory for the building info based on the number above
        *ppClassInfos = new T[*iNumVals];

        //OutputDebugString(CvString::format("[Jason] %s\t0x%x\n", szTagName, *ppClassInfos));
		// set the local pointer to the memory we just allocated
		pClassInfo = *ppClassInfos;

		// loop through each tag
		for (i=0;i<*iNumVals;i++)
		{
			SkipToNextVal();	// skip to the next non-comment node

			if (!pClassInfo[i].read(this))
			{
				break;
			}
			GC.setInfoTypeFromString(pClassInfo[i].getType(), i);	// add type to global info type hash map
			if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
			{
				break;
			}
		}
	}

	// if we didn't find the tag name in the xml then we never set the pszString to the
	// newly allocated memory and there for we will FAssert to let people know this most
	// interesting fact
	if(!pClassInfo)
	{
		char	szMessage[1024];
		sprintf( szMessage, "Error locating tag node in SetGlobalClassInfo function \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
}

//
// helper sort predicate
//

struct OrderIndex {int m_iPriority; int m_iIndex;};
bool sortHotkeyPriority(const OrderIndex orderIndex1, const OrderIndex orderIndex2)
{
	return (orderIndex1.m_iPriority > orderIndex2.m_iPriority);
}

template <class T>
void CvXMLLoadUtility::orderHotkeyInfo(int** ppiSortedIndex, T* pHotkeyInfos, int iLength)
{
	int iI;
	int* piSortedIndex;
	std::vector<OrderIndex> viOrderPriority;

	viOrderPriority.resize(iLength);
	piSortedIndex = *ppiSortedIndex;

	// set up vector
	for(iI=0;iI<iLength;iI++)
	{
		viOrderPriority[iI].m_iPriority = pHotkeyInfos[iI].getOrderPriority();
		viOrderPriority[iI].m_iIndex = iI;
	}

	// sort the array
	std::sort(viOrderPriority.begin(), viOrderPriority.end(), sortHotkeyPriority);

	// insert new order into the array to return
	for (iI=0;iI<iLength;iI++)
	{
		piSortedIndex[iI] = viOrderPriority[iI].m_iIndex;
	}
}

void CvXMLLoadUtility::orderHotkeyInfo(int** ppiSortedIndex, int* pHotkeyIndex, int iLength)
{
	int iI;
	int* piSortedIndex;
	std::vector<OrderIndex> viOrderPriority;

	viOrderPriority.resize(iLength);
	piSortedIndex = *ppiSortedIndex;

	// set up vector
	for(iI=0;iI<iLength;iI++)
	{
		viOrderPriority[iI].m_iPriority = pHotkeyIndex[iI];
		viOrderPriority[iI].m_iIndex = iI;
	}

	// sort the array
	std::sort(viOrderPriority.begin(), viOrderPriority.end(), sortHotkeyPriority);

	// insert new order into the array to return
	for (iI=0;iI<iLength;iI++)
	{
		piSortedIndex[iI] = viOrderPriority[iI].m_iIndex;
	}
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetYields(int** ppiYield)
//
//  PURPOSE :   Allocate memory for the yield parameter and set it to the values
//				in the xml file.  The current/last located node must be the first child of the
//				yield changes node
//
//------------------------------------------------------------------------------------------------------
int CvXMLLoadUtility::SetYields(int** ppiYield)
{
	int i=0;			//loop counter
	int iNumSibs=0;		// the number of siblings the current xml node has
	int *piYield;	// local pointer for the yield memory

	// Skip any comments and stop at the next value we might want
	if (SkipToNextVal())
	{
		// get the total number of children the current xml node has
		iNumSibs = gDLL->getXMLIFace()->GetNumChildren(m_pFXml);

		InitList(ppiYield, NUM_YIELD_TYPES);

		// set the local pointer to the memory we just allocated
		piYield = *ppiYield;

		if (0 < iNumSibs)
		{
			// if the call to the function that sets the current xml node to it's first non-comment
			// child and sets the parameter with the new node's value succeeds
			if (GetChildXmlVal(&piYield[0]))
			{
				if(!(iNumSibs <= NUM_YIELD_TYPES))
				{
					char	szMessage[1024];
					sprintf( szMessage, "For loop iterator is greater than array size \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
					gDLL->MessageBox(szMessage, "XML Error");
				}
				// loop through all the siblings, we start at 1 since we already have the first value
				for (i=1;i<iNumSibs;i++)
				{
					// if the call to the function that sets the current xml node to it's first non-comment
					// sibling and sets the parameter with the new node's value does not succeed
					// we will break out of this for loop
					if (!GetNextXmlVal(&piYield[i]))
					{
						break;
					}
				}
				// set the current xml node to it's parent node
				gDLL->getXMLIFace()->SetToParent(m_pFXml);
			}
		}
	}

	return iNumSibs;
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetFeatureStruct(int** ppiFeatureTech, int** ppiFeatureTime, int** ppiFeatureProduction, bool** ppbFeatureRemove)
//
//  PURPOSE :   allocate and set the feature struct variables for the CvBuildInfo class
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetFeatureStruct(int** ppiFeatureTech, int** ppiFeatureTime, int** ppiFeatureProduction, bool** ppbFeatureRemove)
{
	int i=0;				//loop counter
	int iNumSibs;					// the number of siblings the current xml node has
	int iFeatureIndex;
	TCHAR szTextVal[256];	// temporarily hold the text value of the current xml node
	int* paiFeatureTech = NULL;
	int* paiFeatureTime = NULL;
	int* paiFeatureProduction = NULL;
	bool* pabFeatureRemove = NULL;

	if(GC.getNumFeatureInfos() < 1)
	{
		char	szMessage[1024];
		sprintf( szMessage, "no feature infos set yet! \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	InitList(ppiFeatureTech, GC.getNumFeatureInfos(), -1);
	InitList(ppiFeatureTime, GC.getNumFeatureInfos());
	InitList(ppiFeatureProduction, GC.getNumFeatureInfos());
	InitList(ppbFeatureRemove, GC.getNumFeatureInfos());

	paiFeatureTech = *ppiFeatureTech;
	paiFeatureTime = *ppiFeatureTime;
	paiFeatureProduction = *ppiFeatureProduction;
	pabFeatureRemove = *ppbFeatureRemove;

	if (gDLL->getXMLIFace()->SetToChildByTagName(m_pFXml,"FeatureStructs"))
	{
		iNumSibs = gDLL->getXMLIFace()->GetNumChildren(m_pFXml);

		if (0 < iNumSibs)
		{
			if (gDLL->getXMLIFace()->SetToChildByTagName(m_pFXml,"FeatureStruct"))
			{
				if(!(iNumSibs <= GC.getNumFeatureInfos()))
				{
					char	szMessage[1024];
					sprintf( szMessage, "iNumSibs is greater than GC.getNumFeatureInfos in SetFeatureStruct function \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
					gDLL->MessageBox(szMessage, "XML Error");
				}
				for (i=0;i<iNumSibs;i++)
				{
					GetChildXmlValByName(szTextVal, "FeatureType");
					iFeatureIndex = FindInInfoClass(szTextVal, GC.getFeatureInfo(), sizeof(GC.getFeatureInfo((FeatureTypes) 0)), GC.getNumFeatureInfos());
					if(!(iFeatureIndex != -1))
					{
						char	szMessage[1024];
						sprintf( szMessage, "iFeatureIndex is -1 inside SetFeatureStruct function \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
						gDLL->MessageBox(szMessage, "XML Error");
					}
					GetChildXmlValByName(szTextVal, "PrereqTech");
					paiFeatureTech[iFeatureIndex] = FindInInfoClass(szTextVal, GC.getTechInfo(), sizeof(GC.getTechInfo((TechTypes) 0)), GC.getNumTechInfos());
					GetChildXmlValByName(&paiFeatureTime[iFeatureIndex], "iTime");
					GetChildXmlValByName(&paiFeatureProduction[iFeatureIndex], "iProduction");
					GetChildXmlValByName(&pabFeatureRemove[iFeatureIndex], "bRemove");

					if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
					{
						break;
					}
				}

				gDLL->getXMLIFace()->SetToParent(m_pFXml);
			}
		}

		gDLL->getXMLIFace()->SetToParent(m_pFXml);
	}
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetImprovementBonuses(CvImprovementBonusInfo** ppImprovementBonus)
//
//  PURPOSE :   Allocate memory for the improvement bonus pointer and fill it based on the
//				values in the xml.
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetImprovementBonuses(CvImprovementBonusInfo** ppImprovementBonus)
{
	int i=0;				//loop counter
	int j=0;				//loop counter
	int iNumSibs;			// the number of siblings the current xml node has
	TCHAR szNodeVal[256];	// temporarily holds the string value of the current xml node
	CvImprovementBonusInfo* paImprovementBonus;	// local pointer to the bonus type struct in memory

	// Skip any comments and stop at the next value we might want
	if (SkipToNextVal())
	{
		// initialize the boolean list to the correct size and all the booleans to false
		InitImprovementBonusList(ppImprovementBonus, GC.getNumBonusInfos());
		// set the local pointer to the memory we just allocated
		paImprovementBonus = *ppImprovementBonus;

		// get the total number of children the current xml node has
		iNumSibs = gDLL->getXMLIFace()->GetNumChildren(m_pFXml);
		// if we can set the current xml node to the child of the one it is at now
		if (gDLL->getXMLIFace()->SetToChild(m_pFXml))
		{
			if(!(iNumSibs <= GC.getNumBonusInfos()))
			{
				char	szMessage[1024];
				sprintf( szMessage, "For loop iterator is greater than array size \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
				gDLL->MessageBox(szMessage, "XML Error");
			}
			// loop through all the siblings
			for (i=0;i<iNumSibs;i++)
			{
				// skip to the next non-comment node
				if (SkipToNextVal())
				{
					// call the function that sets the FXml pointer to the first non-comment child of
					// the current tag and gets the value of that new node
					if (GetChildXmlVal(szNodeVal))
					{
						int iBonusIndex;	// index of the match in the bonus types list
						// call the find in list function to return either -1 if no value is found
						// or the index in the list the match is found at
						iBonusIndex = FindInInfoClass(szNodeVal, GC.getBonusInfo(), sizeof(GC.getBonusInfo((BonusTypes) 0)), GC.getNumBonusInfos());
						// if we found a match we will get the next sibling's boolean value at that match's index
						if (iBonusIndex >= 0)
						{
							GetNextXmlVal(&paImprovementBonus[iBonusIndex].m_bBonusMakesValid);
							GetNextXmlVal(&paImprovementBonus[iBonusIndex].m_bBonusTrade);
							GetNextXmlVal(&paImprovementBonus[iBonusIndex].m_iDiscoverRand);
							gDLL->getXMLIFace()->SetToParent(m_pFXml);

							SAFE_DELETE_ARRAY(paImprovementBonus[iBonusIndex].m_piYieldChange);	// free memory - MT, since we are about to reallocate
							if (gDLL->getXMLIFace()->SetToChildByTagName(m_pFXml,"YieldChanges"))
							{
								SetYields(&paImprovementBonus[iBonusIndex].m_piYieldChange);
								gDLL->getXMLIFace()->SetToParent(m_pFXml);
							}
							else
							{
								InitList(&paImprovementBonus[iBonusIndex].m_piYieldChange, NUM_YIELD_TYPES);
							}
						}
						else
						{
							gDLL->getXMLIFace()->SetToParent(m_pFXml);
						}

						// set the current xml node to it's parent node
					}

					// if we cannot set the current xml node to it's next sibling then we will break out of the for loop
					// otherwise we will continue looping
					if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
					{
						break;
					}
				}
			}
			// set the current xml node to it's parent node
			gDLL->getXMLIFace()->SetToParent(m_pFXml);
		}
	}
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetAndLoadVar(int** ppiVar, int iDefault)
//
//  PURPOSE :   set the variable to a default and load it from the xml if there are any children
//
//------------------------------------------------------------------------------------------------------
bool CvXMLLoadUtility::SetAndLoadVar(int** ppiVar, int iDefault)
{
	int iNumSibs;
	int* piVar;
	bool bReturn = false;
	int i; // loop counter

	// Skip any comments and stop at the next value we might want
	if (SkipToNextVal())
	{
		bReturn = true;

		// get the total number of children the current xml node has
		iNumSibs = gDLL->getXMLIFace()->GetNumChildren(m_pFXml);

		// allocate memory
		InitList(ppiVar, iNumSibs, iDefault);

		// set the a local pointer to the newly allocated memory
		piVar = *ppiVar;

		// if the call to the function that sets the current xml node to it's first non-comment
		// child and sets the parameter with the new node's value succeeds
		if (GetChildXmlVal(&piVar[0]))
		{
			// loop through all the siblings, we start at 1 since we already got the first sibling
			for (i=1;i<iNumSibs;i++)
			{
				// if the call to the function that sets the current xml node to it's next non-comment
				// sibling and sets the parameter with the new node's value does not succeed
				// we will break out of this for loop
				if (!GetNextXmlVal(&piVar[i]))
				{
					break;
				}
			}

			// set the current xml node to it's parent node
			gDLL->getXMLIFace()->SetToParent(m_pFXml);
		}
	}

	return bReturn;
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetVariableListTagPair(	int **ppiList, const TCHAR* szRootTagName,
//										CvInfoBase* pCvInfoBase,
//										int iInfoBaseSize, int iInfoBaseLength, int iDefaultListVal)
//
//  PURPOSE :   allocate and initialize a list from a tag pair in the xml
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetVariableListTagPair(int **ppiList, const TCHAR* szRootTagName,
											  CvInfoBase* pCvInfoBase,
											  int iInfoBaseSize, int iInfoBaseLength, int iDefaultListVal)
{
	int i;
	int iIndexVal;
	int iNumSibs;
	TCHAR szTextVal[256];
	int* piList;

	if(!(0 < iInfoBaseLength))
	{
		char	szMessage[1024];
		sprintf( szMessage, "Allocating zero or less memory in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	InitList(ppiList, iInfoBaseLength, iDefaultListVal);
	if (gDLL->getXMLIFace()->SetToChildByTagName(m_pFXml,szRootTagName))
	{
		if (SkipToNextVal())
		{
			iNumSibs = gDLL->getXMLIFace()->GetNumChildren(m_pFXml);
			piList = *ppiList;
			if (0 < iNumSibs)
			{
				if(!(iNumSibs <= iInfoBaseLength))
				{
					char	szMessage[1024];
					sprintf( szMessage, "There are more siblings than memory allocated for them in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
					gDLL->MessageBox(szMessage, "XML Error");
				}
				if (gDLL->getXMLIFace()->SetToChild(m_pFXml))
				{
					for (i=0;i<iNumSibs;i++)
					{
						if (GetChildXmlVal(szTextVal))
						{
							iIndexVal = FindInInfoClass(szTextVal, pCvInfoBase, iInfoBaseSize, iInfoBaseLength);
							GetNextXmlVal(&piList[iIndexVal]);

							gDLL->getXMLIFace()->SetToParent(m_pFXml);
						}

						if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
						{
							break;
						}
					}

					gDLL->getXMLIFace()->SetToParent(m_pFXml);
				}
			}
		}

		gDLL->getXMLIFace()->SetToParent(m_pFXml);
	}
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetVariableListTagPair(	bool **ppbList, const TCHAR* szRootTagName,
//										CvInfoBase* pCvInfoBase,
//										int iInfoBaseSize, int iInfoBaseLength, bool bDefaultListVal)
//
//  PURPOSE :   allocate and initialize a list from a tag pair in the xml
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetVariableListTagPair(bool **ppbList, const TCHAR* szRootTagName,
											  CvInfoBase* pCvInfoBase,
											  int iInfoBaseSize, int iInfoBaseLength, bool bDefaultListVal)
{
	int i;
	int iIndexVal;
	int iNumSibs;
	TCHAR szTextVal[256];
	bool* pbList;

	if(!(0 < iInfoBaseLength))
	{
		char	szMessage[1024];
		sprintf( szMessage, "Allocating zero or less memory in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	InitList(ppbList, iInfoBaseLength, bDefaultListVal);
	if (gDLL->getXMLIFace()->SetToChildByTagName(m_pFXml,szRootTagName))
	{
		if (SkipToNextVal())
		{
			iNumSibs = gDLL->getXMLIFace()->GetNumChildren(m_pFXml);
			pbList = *ppbList;
			if (0 < iNumSibs)
			{
				if(!(iNumSibs <= iInfoBaseLength))
				{
					char	szMessage[1024];
					sprintf( szMessage, "There are more siblings than memory allocated for them in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
					gDLL->MessageBox(szMessage, "XML Error");
				}
				if (gDLL->getXMLIFace()->SetToChild(m_pFXml))
				{
					for (i=0;i<iNumSibs;i++)
					{
						if (GetChildXmlVal(szTextVal))
						{
							iIndexVal = FindInInfoClass(szTextVal, pCvInfoBase, iInfoBaseSize, iInfoBaseLength);
							GetNextXmlVal(&pbList[iIndexVal]);

							gDLL->getXMLIFace()->SetToParent(m_pFXml);
						}

						if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
						{
							break;
						}
					}

					gDLL->getXMLIFace()->SetToParent(m_pFXml);
				}
			}
		}

		gDLL->getXMLIFace()->SetToParent(m_pFXml);
	}
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetVariableListTagPair(	float **ppfList, const TCHAR* szRootTagName,
//										CvInfoBase* pCvInfoBase,
//										int iInfoBaseSize, int iInfoBaseLength, float fDefaultListVal)
//
//  PURPOSE :   allocate and initialize a list from a tag pair in the xml
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetVariableListTagPair(float **ppfList, const TCHAR* szRootTagName,
											  CvInfoBase* pCvInfoBase,
											  int iInfoBaseSize, int iInfoBaseLength, float fDefaultListVal)
{
	int i;
	int iIndexVal;
	int iNumSibs;
	TCHAR szTextVal[256];
	float* pfList;

	if(!(0 < iInfoBaseLength))
	{
		char	szMessage[1024];
		sprintf( szMessage, "Allocating zero or less memory in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	InitList(ppfList, iInfoBaseLength, fDefaultListVal);
	if (gDLL->getXMLIFace()->SetToChildByTagName(m_pFXml,szRootTagName))
	{
		if (SkipToNextVal())
		{
			iNumSibs = gDLL->getXMLIFace()->GetNumChildren(m_pFXml);
			pfList = *ppfList;
			if (0 < iNumSibs)
			{
				if(!(iNumSibs <= iInfoBaseLength))
				{
					char	szMessage[1024];
					sprintf( szMessage, "There are more siblings than memory allocated for them in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
					gDLL->MessageBox(szMessage, "XML Error");
				}
				if (gDLL->getXMLIFace()->SetToChild(m_pFXml))
				{
					for (i=0;i<iNumSibs;i++)
					{
						if (GetChildXmlVal(szTextVal))
						{
							iIndexVal = FindInInfoClass(szTextVal, pCvInfoBase, iInfoBaseSize, iInfoBaseLength);
							GetNextXmlVal(&pfList[iIndexVal]);

							gDLL->getXMLIFace()->SetToParent(m_pFXml);
						}

						if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
						{
							break;
						}
					}

					gDLL->getXMLIFace()->SetToParent(m_pFXml);
				}
			}
		}

		gDLL->getXMLIFace()->SetToParent(m_pFXml);
	}
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetVariableListTagPair(	CvString **ppfList, const TCHAR* szRootTagName,
//										CvInfoBase* pCvInfoBase,
//										int iInfoBaseSize, int iInfoBaseLength, CvString szDefaultListVal)
//
//  PURPOSE :   allocate and initialize a list from a tag pair in the xml
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetVariableListTagPair(CvString **ppszList, const TCHAR* szRootTagName,
											  CvInfoBase* pCvInfoBase,
											  int iInfoBaseSize, int iInfoBaseLength, CvString szDefaultListVal)
{
	int i;
	int iIndexVal;
	int iNumSibs;
	TCHAR szTextVal[256];
	CvString* pszList;

	if(!(0 < iInfoBaseLength))
	{
		char	szMessage[1024];
		sprintf( szMessage, "Allocating zero or less memory in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	InitStringList(ppszList, iInfoBaseLength, szDefaultListVal);
	if (gDLL->getXMLIFace()->SetToChildByTagName(m_pFXml,szRootTagName))
	{
		if (SkipToNextVal())
		{
			iNumSibs = gDLL->getXMLIFace()->GetNumChildren(m_pFXml);
			pszList = *ppszList;
			if (0 < iNumSibs)
			{
				if(!(iNumSibs <= iInfoBaseLength))
				{
					char	szMessage[1024];
					sprintf( szMessage, "There are more siblings than memory allocated for them in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
					gDLL->MessageBox(szMessage, "XML Error");
				}
				if (gDLL->getXMLIFace()->SetToChild(m_pFXml))
				{
					for (i=0;i<iNumSibs;i++)
					{
						if (GetChildXmlVal(szTextVal))
						{
							iIndexVal = FindInInfoClass(szTextVal, pCvInfoBase, iInfoBaseSize, iInfoBaseLength);
							GetNextXmlVal(pszList[iIndexVal]);

							gDLL->getXMLIFace()->SetToParent(m_pFXml);
						}

						if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
						{
							break;
						}
					}

					gDLL->getXMLIFace()->SetToParent(m_pFXml);
				}
			}
		}

		gDLL->getXMLIFace()->SetToParent(m_pFXml);
	}
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetVariableListTagPair(int **ppiList, const TCHAR* szRootTagName,
//										CvString* m_paszTagList, int iTagListLength, int iDefaultListVal)
//
//  PURPOSE :   allocate and initialize a list from a tag pair in the xml
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetVariableListTagPair(int **ppiList, const TCHAR* szRootTagName,
											  CvString* m_paszTagList, int iTagListLength, int iDefaultListVal)
{
	int i;
	int iIndexVal;
	int iNumSibs;
	TCHAR szTextVal[256];
	int* piList;

	if(!(0 < iTagListLength))
	{
		char	szMessage[1024];
		sprintf( szMessage, "Allocating zero or less memory in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	InitList(ppiList, iTagListLength, iDefaultListVal);
	if (gDLL->getXMLIFace()->SetToChildByTagName(m_pFXml,szRootTagName))
	{
		if (SkipToNextVal())
		{
			iNumSibs = gDLL->getXMLIFace()->GetNumChildren(m_pFXml);
			piList = *ppiList;
			if (0 < iNumSibs)
			{
				if(!(iNumSibs <= iTagListLength))
				{
					char	szMessage[1024];
					sprintf( szMessage, "There are more siblings than memory allocated for them in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
					gDLL->MessageBox(szMessage, "XML Error");
				}
				if (gDLL->getXMLIFace()->SetToChild(m_pFXml))
				{
					for (i=0;i<iNumSibs;i++)
					{
						if (GetChildXmlVal(szTextVal))
						{
							iIndexVal = GC.getTypesEnum(szTextVal);
							GetNextXmlVal(&piList[iIndexVal]);

							gDLL->getXMLIFace()->SetToParent(m_pFXml);
						}

						if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
						{
							break;
						}
					}

					gDLL->getXMLIFace()->SetToParent(m_pFXml);
				}
			}
		}

		gDLL->getXMLIFace()->SetToParent(m_pFXml);
	}
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetVariableListTagPair(int **ppiList, const TCHAR* szRootTagName,
//										CvString* m_paszTagList, int iTagListLength, int iDefaultListVal)
//
//  PURPOSE :   allocate and initialize a list from a tag pair in the xml for audio scripts
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetVariableListTagPairForAudioScripts(int **ppiList, const TCHAR* szRootTagName,
															 CvString* m_paszTagList, int iTagListLength, int iDefaultListVal)
{
	int i;
	int iIndexVal;
	int iNumSibs;
	TCHAR szTextVal[256];
	int* piList;
	CvString szTemp;

	if (gDLL->getXMLIFace()->SetToChildByTagName(m_pFXml,szRootTagName))
	{
		if (SkipToNextVal())
		{
			iNumSibs = gDLL->getXMLIFace()->GetNumChildren(m_pFXml);
			if(!(0 < iTagListLength))
			{
				char	szMessage[1024];
				sprintf( szMessage, "Allocating zero or less memory in CvXMLLoadUtility::SetVariableListTagPairForAudio \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
				gDLL->MessageBox(szMessage, "XML Error");
			}
			InitList(ppiList, iTagListLength, iDefaultListVal);
			piList = *ppiList;
			if (0 < iNumSibs)
			{
				if(!(iNumSibs <= iTagListLength))
				{
					char	szMessage[1024];
					sprintf( szMessage, "There are more siblings than memory allocated for them in CvXMLLoadUtility::SetVariableListTagPairForAudio \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
					gDLL->MessageBox(szMessage, "XML Error");
				}
				if (gDLL->getXMLIFace()->SetToChild(m_pFXml))
				{
					for (i=0;i<iNumSibs;i++)
					{
						if (GetChildXmlVal(szTextVal))
						{
							iIndexVal =	GC.getTypesEnum(szTextVal);
							GetNextXmlVal(szTemp);
							if ( szTemp.GetLength() > 0 )
								piList[iIndexVal] = gDLL->getAudioTagIndex(szTemp);
							else
								piList[iIndexVal] = -1;

							gDLL->getXMLIFace()->SetToParent(m_pFXml);
						}

						if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
						{
							break;
						}
					}

					gDLL->getXMLIFace()->SetToParent(m_pFXml);
				}
			}
		}

		gDLL->getXMLIFace()->SetToParent(m_pFXml);
	}
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetVariableListTagPairForAudioScripts(int **ppiList, const TCHAR* szRootTagName,
//										CvInfoBase* pCvInfoBase, int iInfoBaseSize, int iInfoBaseLength, int iDefaultListVal)
//
//  PURPOSE :   allocate and initialize a list from a tag pair in the xml for audio scripts
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetVariableListTagPairForAudioScripts(int **ppiList, const TCHAR* szRootTagName,
															 CvInfoBase* pCvInfoBase, int iInfoBaseSize, int iInfoBaseLength, int iDefaultListVal)
{
	int i;
	int iIndexVal;
	int iNumSibs;
	TCHAR szTextVal[256];
	int* piList;
	CvString szTemp;

	if (gDLL->getXMLIFace()->SetToChildByTagName(m_pFXml,szRootTagName))
	{
		if (SkipToNextVal())
		{
			iNumSibs = gDLL->getXMLIFace()->GetNumChildren(m_pFXml);
			if(!(0 < iInfoBaseLength))
			{
				char	szMessage[1024];
				sprintf( szMessage, "Allocating zero or less memory in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
				gDLL->MessageBox(szMessage, "XML Error");
			}
			InitList(ppiList, iInfoBaseLength, iDefaultListVal);
			piList = *ppiList;
			if (0 < iNumSibs)
			{
				if(!(iNumSibs <= iInfoBaseLength))
				{
					char	szMessage[1024];
					sprintf( szMessage, "There are more siblings than memory allocated for them in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
					gDLL->MessageBox(szMessage, "XML Error");
				}
				if (gDLL->getXMLIFace()->SetToChild(m_pFXml))
				{
					for (i=0;i<iNumSibs;i++)
					{
						if (GetChildXmlVal(szTextVal))
						{
							iIndexVal = FindInInfoClass(szTextVal, pCvInfoBase, iInfoBaseSize, iInfoBaseLength);
							GetNextXmlVal(szTemp);
							if ( szTemp.GetLength() > 0 )
								piList[iIndexVal] = gDLL->getAudioTagIndex(szTemp);
							else
								piList[iIndexVal] = -1;

							gDLL->getXMLIFace()->SetToParent(m_pFXml);
						}

						if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
						{
							break;
						}
					}

					gDLL->getXMLIFace()->SetToParent(m_pFXml);
				}
			}
		}

		gDLL->getXMLIFace()->SetToParent(m_pFXml);
	}
}

//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetVariableListTagPair(bool **ppbList, const TCHAR* szRootTagName,
//										CvString* m_paszTagList, int iTagListLength, int iDefaultListVal)
//
//  PURPOSE :   allocate and initialize a list from a tag pair in the xml
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetVariableListTagPair(bool **ppbList, const TCHAR* szRootTagName,
											  CvString* m_paszTagList, int iTagListLength, bool bDefaultListVal)
{
	int i;
	int iIndexVal;
	int iNumSibs;
	TCHAR szTextVal[256];
	bool* pbList;

	if(!(0 < iTagListLength))
	{
		char	szMessage[1024];
		sprintf( szMessage, "Allocating zero or less memory in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	InitList(ppbList, iTagListLength, bDefaultListVal);
	if (gDLL->getXMLIFace()->SetToChildByTagName(m_pFXml,szRootTagName))
	{
		if (SkipToNextVal())
		{
			iNumSibs = gDLL->getXMLIFace()->GetNumChildren(m_pFXml);
			pbList = *ppbList;
			if (0 < iNumSibs)
			{
				if(!(iNumSibs <= iTagListLength))
				{
					char	szMessage[1024];
					sprintf( szMessage, "There are more siblings than memory allocated for them in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
					gDLL->MessageBox(szMessage, "XML Error");
				}
				if (gDLL->getXMLIFace()->SetToChild(m_pFXml))
				{
					for (i=0;i<iNumSibs;i++)
					{
						if (GetChildXmlVal(szTextVal))
						{
							iIndexVal =	GC.getTypesEnum(szTextVal);
							GetNextXmlVal(&pbList[iIndexVal]);

							gDLL->getXMLIFace()->SetToParent(m_pFXml);
						}

						if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
						{
							break;
						}
					}

					gDLL->getXMLIFace()->SetToParent(m_pFXml);
				}
			}
		}

		gDLL->getXMLIFace()->SetToParent(m_pFXml);
	}
}

//------------------------------------------------------------------------------------------------------
//
//	FUNCTION:	SetVariableListTagPair(CvString **ppszList, const TCHAR* szRootTagName,
//							CvString* m_paszTagList, int iTagListLength, CvString szDefaultListVal = "")
//
//  PURPOSE :   allocate and initialize a list from a tag pair in the xml
//
//------------------------------------------------------------------------------------------------------
void CvXMLLoadUtility::SetVariableListTagPair(CvString **ppszList, const TCHAR* szRootTagName, CvString* m_paszTagList, int iTagListLength, CvString szDefaultListVal)
{
	int i;
	int iIndexVal;
	int iNumSibs;
	TCHAR szTextVal[256];
	CvString* pszList;

	if(!(0 < iTagListLength))
	{
		char	szMessage[1024];
		sprintf( szMessage, "Allocating zero or less memory in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	InitStringList(ppszList, iTagListLength, szDefaultListVal);
	if (gDLL->getXMLIFace()->SetToChildByTagName(m_pFXml,szRootTagName))
	{
		if (SkipToNextVal())
		{
			iNumSibs = gDLL->getXMLIFace()->GetNumChildren(m_pFXml);
			pszList = *ppszList;
			if (0 < iNumSibs)
			{
				if(!(iNumSibs <= iTagListLength))
				{
					char	szMessage[1024];
					sprintf( szMessage, "There are more siblings than memory allocated for them in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
					gDLL->MessageBox(szMessage, "XML Error");
				}
				if (gDLL->getXMLIFace()->SetToChild(m_pFXml))
				{
					for (i=0;i<iNumSibs;i++)
					{
						if (GetChildXmlVal(szTextVal))
						{
							iIndexVal =	GC.getTypesEnum(szTextVal);
							GetNextXmlVal(pszList[iIndexVal]);

							gDLL->getXMLIFace()->SetToParent(m_pFXml);
						}

						if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
						{
							break;
						}
					}

					gDLL->getXMLIFace()->SetToParent(m_pFXml);
				}
			}
		}

		gDLL->getXMLIFace()->SetToParent(m_pFXml);
	}
}

DllExport bool CvXMLLoadUtility::LoadPlayerOptions()
{
	if (!CreateFXml())
		return false;

	bool bLoaded = LoadCivXml(m_pFXml, "GameInfo/CIV4PlayerOptionInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for GameInfo/CIV4PlayerOptionInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	if (bLoaded && Validate())
	{
		int iEnumVal = NUM_PLAYEROPTION_TYPES;
		SetGlobalClassInfo(&GC.getPlayerOptionInfo(), "Civ4PlayerOptionInfos/PlayerOptionInfos/PlayerOptionInfo", &iEnumVal, true);
	}
	DestroyFXml();
	return bLoaded;
}

DllExport bool CvXMLLoadUtility::LoadGraphicOptions()
{
	if (!CreateFXml())
		return false;

	bool bLoaded = LoadCivXml(m_pFXml, "GameInfo/CIV4GraphicOptionInfos.xml");
	if (!bLoaded)
	{
		char	szMessage[1024];
		sprintf( szMessage, "LoadXML call failed for GameInfo/CIV4GraphicOptionInfos.xml. \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
	if (bLoaded && Validate())
	{
		int iEnumVal = NUM_GRAPHICOPTION_TYPES;
		SetGlobalClassInfo(&GC.getGraphicOptionInfo(), "Civ4GraphicOptionInfos/GraphicOptionInfos/GraphicOptionInfo", &iEnumVal, true);
	}
	DestroyFXml();
	return bLoaded;
}

//----------------------MODULAR LOADING FUNCTIONS-----------------------------

template <class T>  // Modular Loading by Chinese American
void CvXMLLoadUtility::AddGlobalArtDefineInfo(T **ppArtDefneInfos, char* szXMLFileName, char* szTagName, int& iNumVals)
{
    //
    // load extra files in the xml directory
    //
    // szXMLFileName is pattern to files names, eg "xml\\Art\\*_CIV4ArtDefines_Unit.xml"
    std::vector<CvString> files;
    gDLL->enumerateFiles(files, szXMLFileName);
    int i;
    for(i=0;i<(int)files.size();i++)
    {
        if (!CreateFXml())
            return;

        bool bLoaded = LoadCivXml(m_pFXml, files[i]);	// Load each XML
        if (!bLoaded)
        {
            char	szMessage[1024];
            sprintf( szMessage, "LoadXML call failed for current XML file:\n %s", GC.getCurrentXMLFile().GetCString());
            gDLL->MessageBox(szMessage, "XML Load Error");
        }

        if (bLoaded && Validate())
        {
            AddGlobalClassInfo(&*ppArtDefneInfos, szTagName, &iNumVals);
        }
        DestroyFXml();
    }
}

template <class T>  // Modular Loading by Chinese American
void CvXMLLoadUtility::AddGlobalClassInfo(T **ppClassInfos, char* szTagName, int* iNumVals)
{
	char szLog[256];
	sprintf(szLog, "AddGlobalClassInfo (%s)", szTagName);
	PROFILE(szLog);
	logMsg(szLog);

	int i=0;					//loop counter
	T* pClassInfo = NULL;	// local pointer to the building info memory
	int iOldNumVals = *iNumVals;    // original array size
	int iTypeIndex;        // start index for new types

	// if we successfully locate the tag name in the xml file
	if (gDLL->getXMLIFace()->LocateNode(m_pFXml,szTagName))
	{
        // get the number of times the szTagName tag appears in the xml file
        *iNumVals += gDLL->getXMLIFace()->NumOfElementsByTagName(m_pFXml,szTagName);
	}

    // new temp array
	T *ppTempClassInfos = new T[*iNumVals];

    //OutputDebugString(CvString::format("[Jason] %s\t0x%x\n", szTagName, *ppClassInfos));
    // set the local pointer to the memory we just allocated
    pClassInfo = *ppClassInfos;

    for (int j=0;j<iOldNumVals;j++)
    {
        ppTempClassInfos[j] = pClassInfo[j];
    }
    *ppClassInfos = ppTempClassInfos;
    pClassInfo = *ppClassInfos;

    // loop through each tag
    iTypeIndex = iOldNumVals;
    for (i=iOldNumVals;i<(*iNumVals);i++)
    {
        SkipToNextVal();	// skip to the next non-comment node

        if (!pClassInfo[iTypeIndex].read(this))
        {
            break;
        }
        int iInfoTypeIndex = GC.getInfoTypeForString(pClassInfo[iTypeIndex].getType());
        if ( iInfoTypeIndex == -1 )
        {
            sprintf(szLog, "Adding %s to %s Info at Index %i", pClassInfo[iTypeIndex].getType(), szTagName, iTypeIndex);
            PROFILE(szLog);
            logMsg(szLog);
            GC.setInfoTypeFromString(pClassInfo[iTypeIndex].getType(), iTypeIndex);	// add type to global info type hash map
            iTypeIndex++;
        }
        else
        {
            sprintf(szLog, "Replacing old %s Entry at Index %i in %s Info", pClassInfo[iTypeIndex].getType(), iInfoTypeIndex, szTagName);
            PROFILE(szLog);
            logMsg(szLog);
            pClassInfo[iInfoTypeIndex] = pClassInfo[iTypeIndex];
            GC.setInfoTypeFromString(pClassInfo[iInfoTypeIndex].getType(), iInfoTypeIndex);	// replace type to global info type hash map
        }
        if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
        {
            break;
        }
    }
    *iNumVals = iTypeIndex;

	// if we didn't find the tag name in the xml then we never set the pszString to the
	// newly allocated memory and there for we will FAssert to let people know this most
	// interesting fact
	if(!pClassInfo)
	{
		char	szMessage[1024];
		sprintf( szMessage, "Error locating tag node in SetGlobalClassInfo function \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}
}

// Modular Loading by Impaler[WrG]
char* CvXMLLoadUtility::MergeStrings(char* format, ... )
{
	static char buf[1024];
	_vsnprintf( buf, 1020, format, (char*)(&format+1) );
	return &buf[0];
}

// Modular Loading by Impaler[WrG]
void CvXMLLoadUtility::InitializeClassLoading(std::vector<CvString>& files, char* FileRoot, char* FileDirectory, char* XmlPath, int* iNumVals)
{
    char szLog[256];
	sprintf(szLog, "Initializing Class Loading (%s)", FileRoot);
	PROFILE(szLog);
	logMsg(szLog);
    bool bLoaded;

	*iNumVals = 0;          // Initialize   //TODO check to see if thats nessary

    if (true /*ADD CONTROL STATMENT HERE*/)  // Global define "SUPRESS_LOADING_"FILEROOT"
    {
        bLoaded = MapClassTypes(MergeStrings("%s/%s.xml", FileDirectory, FileRoot), XmlPath, iNumVals);
        files.push_back(MergeStrings("%s/%s.xml", FileDirectory, FileRoot));
    }
    if(!bLoaded)
    {
        sprintf(szLog, "Type Mapping Failed for file (%s)", MergeStrings("%s/%s.xml", FileDirectory, FileRoot));
        PROFILE(szLog);
        logMsg(szLog);
    }

    std::vector<CvString> tempfiles;
    gDLL->enumerateFiles(tempfiles, MergeStrings("xml\*_%s.xml", FileRoot));  // search for the modular files

    for(int i=0; i<(int)tempfiles.size(); i++)
    {
        bLoaded = MapClassTypes(tempfiles[i], XmlPath, iNumVals);      // Load each XML

        if(!bLoaded)
        {
            sprintf(szLog, "Type Mapping Failed for file (%s)", tempfiles[i]);
            PROFILE(szLog);
            logMsg(szLog);
        }
        else
        {
            files.push_back(tempfiles[i]);  // If the file maped sucessfuly then add it to vector for loading to be completed
        }
    }
}

// Modular Loading by Impaler[WrG]
bool CvXMLLoadUtility::MapClassTypes(const TCHAR* file, char* XmlPath, int* iNumVals)
{
    char szLog[256];
    char szTextVal[256];	// Holds the value of a retrieved key that is to be interpreted
	int iNumNodes;			// nodes found

    bool bLoaded = LoadCivXml(m_pFXml, file);  // Record succesfull loading

    if (!(bLoaded))
    {
        sprintf(szLog, "LoadXML call failed for modular loading \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
        gDLL->MessageBox(szLog, "XML Load Error");
    }
    if (bLoaded && Validate())
    {
        if (gDLL->getXMLIFace()->LocateNode(m_pFXml, XmlPath))
        {
            iNumNodes = gDLL->getXMLIFace()->NumOfElementsByTagName(m_pFXml, XmlPath);
            gDLL->getXMLIFace()->SetToParent(m_pFXml);
            gDLL->getXMLIFace()->SetToChild(m_pFXml);

            for (int i=0; i<iNumNodes; i++)
            {
                SkipToNextVal();	// skip to the next non-comment node
                if (GetChildXmlValByName(szTextVal, "Type"))
                {
                    if (GC.getInfoTypeForString(szTextVal) == -1 )  // It wasnt found so it needs to be added
                    {
                        sprintf(szLog, "Adding %s to Hash map at index %i", szTextVal, *iNumVals);
                        PROFILE(szLog);
                        logMsg(szLog);

                        GC.setInfoTypeFromString(szTextVal, *iNumVals);
                        *iNumVals += 1;
                    }
                }
                if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
                {
                    break;
                }
            }
            gDLL->getXMLIFace()->SetToParent(m_pFXml);
        }
    }
    return bLoaded;
}

template <class T>  // Modular Loading by Impaler[WrG]
void CvXMLLoadUtility::CompleteClassLoading(std::vector<CvString>& files, T **ppClassInfos, char* XmlPath, int* iNumVals)
{
 	char szLog[1024];
	sprintf(szLog, "Completing Class Loading of %i Elements of type (%s)", *iNumVals, XmlPath);
	PROFILE(szLog);
	logMsg(szLog);

    T* pClassInfo = *ppClassInfos;	      // local pointer to the building info memory
    *ppClassInfos = new T[*iNumVals];     // Allocate an array for each tag thats been found

    for(int i=0; i<(int)files.size(); i++)
    {
        CompleteFileLoading(files[i], ppClassInfos, XmlPath);
    }
}


template <class T>  // Modular Loading by Impaler[WrG]
void CvXMLLoadUtility::CompleteCachedClassLoading(std::vector<CvString>& files, T **ppClassInfos, char* XmlPath, int* iNumVals, char* FileDirectory, char* FileRoot, CvCacheObject* (CvDLLUtilityIFaceBase::*pArgFunction) (const TCHAR*))
{
 	char szLog[1024];
	sprintf(szLog, "Completing Class Loading of %i Elements of type (%s)", *iNumVals, XmlPath);
	PROFILE(szLog);
	logMsg(szLog);
    CvCacheObject* cache;
    bool bCacheread;

    cache = (gDLL->*pArgFunction)(MergeStrings("%s.dat", FileRoot));

    logMsg(MergeStrings("xml\\\\%s\\\\%s.xml", FileDirectory, FileRoot));

    bCacheread = gDLL->cacheRead(cache, MergeStrings("xml\\\\%s\\\\%s.xml", FileDirectory, FileRoot));

    if (!bCacheread)
    {
        T* pClassInfo = *ppClassInfos;	      // local pointer to the building info memory
        *ppClassInfos = new T[*iNumVals];     // Allocate an array for each tag thats been found

        for(int i=0; i<(int)files.size(); i++)
        {
            CompleteFileLoading(files[i], ppClassInfos, XmlPath);
        }

        if(!gDLL->cacheWrite(cache))
        {
            sprintf(szLog, "Failed writing to %s.dat cache", FileRoot);
            gDLL->MessageBox(szLog, "XML Caching Error");
        }
        else
        {
            logMsg("Wrote %s to cache", FileRoot);
        }
        gDLL->destroyCache(cache);
    }
    else
    {
        logMsg("Read %s from cache", FileRoot);
    }
}


template <class T>  // Modular Loading by Impaler[WrG]
bool CvXMLLoadUtility::CompleteFileLoading(const TCHAR* file, T **ppClassInfos, char* XmlPath)
{
	char szLog[256];
	sprintf(szLog, "Completing File Loading of (%s)", file);
	PROFILE(szLog);
	logMsg(szLog);
	int i;
	CvString szTextVal;

    T* pClassInfo = *ppClassInfos;
    bool bLoaded = LoadCivXml(m_pFXml, file);

	if (bLoaded) // if the file was loaded
	{
        if(gDLL->getXMLIFace()->LocateNode(m_pFXml, XmlPath))
        {
            while(true) //???
            {
                SkipToNextVal();	// skip to the next non-comment node //TODO update to use local fxml
                if (GetChildXmlValByName(szTextVal, "Type"))
                {
                    i = GC.getInfoTypeForString(szTextVal);
                    if (i != -1)
                    {
                        if (!pClassInfo[i].read(this))
                        {
                            break;
                        }
                        if (!gDLL->getXMLIFace()->NextSibling(m_pFXml))
                        {
                            break;
                        }
                    }
                    else  // Somehow this element wasn't mapped (we should abort here or find a way to substitue the data)
                    {
                        // Perhaps Some kind of array shrinkage could happen here to prevent a dead slot??
                        sprintf(szLog, "Unable to Complete Loading of (%s) it was not found in the global Hash Map", szTextVal);
                        PROFILE(szLog);
                        logMsg(szLog);
                    }
                }
            }
        }
	}
    // if we didn't find the tag name in the xml then we never set the pszString to the
	// newly allocated memory and there for we will FAssert to let people know this most
	// interesting fact
	if(!pClassInfo)
	{
		char	szMessage[1024];
		sprintf( szMessage, "Error locating tag node in CompletFileLoading function \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
	}

	return bLoaded;
}